@@ -25,6 +25,8 @@
 
 #include <cstdint>
 
+enum class PlayerColor : uint8_t;
+
 namespace ICN
 {
     enum : int
@@ -909,7 +911,7 @@ namespace ICN
         YELLOW_FONT,
         YELLOW_SMALLFONT,
         BUTTON_WELL_MAX,
-        BTNBATTLEONLY,
+        BUTTON_BATTLE_ONLY,
         BUTTON_GIFT_GOOD,
         BUTTON_GIFT_EVIL,
         CSLMARKER,
@@ -922,7 +924,6 @@ namespace ICN
         MONSTER_SWITCH_RIGHT_ARROW,
 
         NON_UNIFORM_GOOD_RESTART_BUTTON,
-        NON_UNIFORM_EVIL_RESTART_BUTTON,
 
         UNIFORM_GOOD_MAX_BUTTON,
         UNIFORM_GOOD_MIN_BUTTON,
@@ -1020,12 +1021,11 @@ namespace ICN
         BUTTON_5_PLAYERS,
         BUTTON_6_PLAYERS,
 
-        BUTTON_NEW_GAME_GOOD,
-        BUTTON_NEW_GAME_EVIL,
-        BUTTON_SAVE_GAME_GOOD,
-        BUTTON_SAVE_GAME_EVIL,
-        BUTTON_LOAD_GAME_GOOD,
-        BUTTON_LOAD_GAME_EVIL,
+        BUTTON_NEW_MAP,
+        BUTTON_LOAD_MAP,
+
+        BUTTONS_FILE_DIALOG_GOOD,
+        BUTTONS_FILE_DIALOG_EVIL,
         BUTTON_INFO_GOOD,
         BUTTON_INFO_EVIL,
         BUTTON_QUIT_GOOD,
@@ -1064,18 +1064,9 @@ namespace ICN
         BUTTON_SMALL_MAX_GOOD,
         BUTTON_SMALL_MAX_EVIL,
 
-        BUTTON_EXIT_GOOD,
         BUTTON_RESET_GOOD,
         BUTTON_START_GOOD,
 
-        BUTTON_CASTLE_GOOD,
-        BUTTON_CASTLE_EVIL,
-        BUTTON_TOWN_GOOD,
-        BUTTON_TOWN_EVIL,
-
-        BUTTON_RESTRICT_GOOD,
-        BUTTON_RESTRICT_EVIL,
-
         BUTTON_KINGDOM_EXIT,
         BUTTON_KINGDOM_HEROES,
         BUTTON_KINGDOM_TOWNS,
@@ -1118,18 +1109,6 @@ namespace ICN
         BUTTON_LOAD_MAP_EVIL,
         BUTTON_LOAD_MAP_GOOD,
 
-        BUTTON_RUMORS_GOOD,
-        BUTTON_RUMORS_EVIL,
-        BUTTON_EVENTS_GOOD,
-        BUTTON_EVENTS_EVIL,
-        BUTTON_LANGUAGE_GOOD,
-        BUTTON_LANGUAGE_EVIL,
-
-        BUTTON_AUTO_COMBAT_GOOD,
-        BUTTON_AUTO_COMBAT_EVIL,
-        BUTTON_QUICK_COMBAT_GOOD,
-        BUTTON_QUICK_COMBAT_EVIL,
-
         SCENIBKG_EVIL,
 
         // IMPORTANT! Put any new entry just above this one.
@@ -1144,5 +1123,5 @@ namespace ICN
     int getBuildingIcnId( const int race );
     int getCastleIcnId( const int race );
 
-    int getFlagIcnId( const int color );
+    int getFlagIcnId( const PlayerColor color );
 }
