@@ -27,8 +27,11 @@
 #include <cassert>
 #include <cstdint>
 #include <memory>
+#include <optional>
 #include <ostream>
 #include <string>
+#include <tuple>
+#include <utility>
 #include <vector>
 
 #include "ai_common.h"
@@ -40,10 +43,13 @@
 #include "battle.h"
 #include "castle.h"
 #include "color.h"
+#include "difficulty.h"
 #include "direction.h"
 #include "game.h"
 #include "game_delays.h"
 #include "game_interface.h"
+#include "game_mode.h"
+#include "game_over.h"
 #include "game_static.h"
 #include "heroes.h"
 #include "interface_base.h"
@@ -76,31 +82,37 @@
 
 namespace
 {
-    uint32_t AIGetAllianceColors()
+    PlayerColorsSet AIGetAllianceColors()
     {
         // accumulate colors
-        uint32_t colors = 0;
+        PlayerColorsSet colors = 0;
 
         if ( Settings::Get().GameType() & Game::TYPE_HOTSEAT ) {
-            const Colors vcolors( Players::HumanColors() );
+            const PlayerColorsVector vcolors( Players::HumanColors() );
 
-            for ( const int color : vcolors ) {
+            for ( const PlayerColor color : vcolors ) {
                 const Player * player = Players::Get( color );
-                if ( player )
+                if ( player ) {
                     colors |= player->GetFriends();
+                }
             }
         }
         else {
-            const Player * player = Players::Get( Players::HumanColors() );
-            if ( player )
+            const PlayerColorsSet humanColor = Players::HumanColors();
+
+            assert( Color::Count( humanColor ) == 1 );
+
+            const Player * player = Players::Get( static_cast<PlayerColor>( humanColor ) );
+            if ( player ) {
                 colors = player->GetFriends();
+            }
         }
 
         return colors;
     }
 
     // Never cache the value of this function as it depends on hero's path and location.
-    bool AIIsShowAnimationForHero( const Heroes & hero, const uint32_t colors )
+    bool AIIsShowAnimationForHero( const Heroes & hero, const PlayerColorsSet colors )
     {
         if ( Settings::Get().AIMoveSpeed() == 0 ) {
             return false;
@@ -129,7 +141,7 @@ namespace
         return false;
     }
 
-    bool AIIsShowAnimationForTile( const Maps::Tile & tile, const uint32_t colors )
+    bool AIIsShowAnimationForTile( const Maps::Tile & tile, const PlayerColorsSet colors )
     {
         if ( Settings::Get().AIMoveSpeed() == 0 ) {
             return false;
@@ -234,6 +246,37 @@ namespace
         return true;
     }
 
+    void castGuardianSpellOnCapturedObject( Heroes & hero )
+    {
+        if ( !hero.HaveSpellBook() ) {
+            // The hero doesn't have a spell book.
+            return;
+        }
+
+        const MP2::MapObjectType objectType = hero.getObjectTypeUnderHero();
+        assert( world.getTile( hero.GetIndex() ).getMainObjectType( false ) == objectType );
+
+        // Guardian spells can only be cast on mines.
+        if ( objectType != MP2::OBJ_MINE ) {
+            return;
+        }
+
+        std::vector<Spell> spells = { Spell::SETAGUARDIAN, Spell::SETEGUARDIAN, Spell::SETFGUARDIAN, Spell::SETWGUARDIAN };
+        // Shuffle spells as all of them seem to be equal in power.
+        Rand::Shuffle( spells );
+
+        const int32_t spellMultiplier = Difficulty::getGuardianSpellMultiplier( Game::getDifficulty() );
+
+        for ( const Spell spell : spells ) {
+            if ( hero.CanCastSpell( spell ) && hero.GetSpellPoints() > spellMultiplier * spell.spellPoints( &hero ) ) {
+                // Looks like this hero knows the spell and casting it won't take too many spell points.
+                // So, let's do it!
+                hero.ActionSpellCast( spell );
+                return;
+            }
+        }
+    }
+
     void AITownPortal( Heroes & hero, const int32_t targetIndex )
     {
         assert( Maps::isValidAbsIndex( targetIndex ) );
@@ -344,10 +387,6 @@ namespace
             assert( hero.GetIndex() == dstIndex );
 
             DEBUG_LOG( DBG_AI, DBG_INFO, hero.GetName() << " visits " << castle->GetName() )
-
-            castle->MageGuildEducateHero( hero );
-            hero.SetVisited( dstIndex );
-
             return;
         }
 
@@ -441,9 +480,6 @@ namespace
         DEBUG_LOG( DBG_AI, DBG_INFO, hero.GetName() << " captures the enemy castle " << castle->GetName() )
 
         captureCastle();
-
-        castle->MageGuildEducateHero( hero );
-        hero.SetVisited( dstIndex );
     }
 
     void AIToHeroes( Heroes & hero, const int32_t dstIndex )
@@ -641,73 +677,109 @@ namespace
         DEBUG_LOG( DBG_AI, DBG_INFO, hero.GetName() )
 
         Maps::Tile & tile = world.getTile( dst_index );
-        const Funds funds = getFundsFromTile( tile );
 
-        assert( funds.gold > 0 || funds.GetValidItemsCount() == 0 );
-        uint32_t gold = funds.gold;
+        const auto [goldReward, experienceReward, artifactReward]
+            = [&hero = std::as_const( hero ), objectType,
+               &tile = std::as_const( tile )]() -> std::tuple<std::optional<uint32_t>, std::optional<uint32_t>, std::optional<Artifact>> {
+            const Artifact art = getArtifactFromTile( tile );
+            const Funds funds = getFundsFromTile( tile );
+            assert( funds.gold > 0 || funds.GetValidItemsCount() == 0 );
 
-        Kingdom & kingdom = hero.GetKingdom();
+            uint32_t gold = funds.gold;
 
-        if ( tile.isWater() ) {
-            if ( gold ) {
-                const Artifact & art = getArtifactFromTile( tile );
+            const bool isArtValid = art.isValid();
+            const bool isBagFull = hero.IsFullBagArtifacts();
 
-                if ( art.isValid() && !hero.PickupArtifact( art ) )
+            if ( tile.isWater() ) {
+                if ( gold == 0 ) {
+                    return {};
+                }
+
+                if ( isArtValid && !isBagFull ) {
+                    return { gold, {}, art };
+                }
+
+                if ( isArtValid && isBagFull ) {
                     gold = GoldInsteadArtifact( objectType );
+                }
+
+                return { gold, {}, {} };
             }
-        }
-        else {
-            const Artifact & art = getArtifactFromTile( tile );
 
-            if ( gold ) {
-                const uint32_t experience = gold > 500 ? gold - 500 : 500;
-                const Heroes::Role role = hero.getAIRole();
-                const int32_t kingdomGold = kingdom.GetFunds().gold;
-
-                uint32_t chance = 0;
-                if ( role == Heroes::Role::SCOUT || role == Heroes::Role::COURIER ) {
-                    // These roles usually don't choose experience. Make AI rich!
-                    if ( kingdomGold > 10000 && experience >= 1500 ) {
-                        chance = 10;
-                    }
+            if ( gold > 0 || ( isArtValid && isBagFull ) ) {
+                if ( isArtValid && isBagFull ) {
+                    gold = GoldInsteadArtifact( objectType );
                 }
-                else if ( role == Heroes::Role::CHAMPION ) {
-                    // If AI is extremely low on gold consider taking it
-                    if ( kingdomGold < 3000 ) {
-                        // Safeguard the calculation since we're working with unsigned values
-                        chance = ( std::max( experience, 500U ) - 500 ) / 15;
+
+                const uint32_t exp = gold > 500 ? gold - 500 : 500;
+                const uint32_t chanceToChooseExp = [&hero, exp]() -> uint32_t {
+                    const Heroes::Role role = hero.getAIRole();
+                    const int32_t kingdomGold = hero.GetKingdom().GetFunds().gold;
+
+                    if ( role == Heroes::Role::SCOUT || role == Heroes::Role::COURIER ) {
+                        // These roles usually don't choose experience. Make AI rich!
+                        if ( kingdomGold > 10000 && exp >= 1500 ) {
+                            return 10;
+                        }
+
+                        return 0;
                     }
-                    else {
+
+                    if ( role == Heroes::Role::CHAMPION ) {
+                        // If AI is extremely low on gold consider taking it
+                        if ( kingdomGold < 3000 ) {
+                            // Safeguard the calculation since we're working with unsigned values
+                            return ( std::max( exp, 500U ) - 500 ) / 15;
+                        }
+
                         // Otherwise Champion always picks experience
-                        chance = 100;
+                        return 100;
                     }
-                }
-                else if ( kingdomGold < 3000 ) {
-                    chance = ( role == Heroes::Role::FIGHTER && experience >= 1500 ) ? 10 : 0;
-                }
-                else {
-                    uint32_t value = ( experience > 500 ) ? experience - 500 : 0;
+
+                    if ( kingdomGold < 3000 ) {
+                        return ( role == Heroes::Role::FIGHTER && exp >= 1500 ) ? 10 : 0;
+                    }
+
+                    uint32_t value = std::max( exp, 500U ) - 500;
                     if ( role == Heroes::Role::FIGHTER ) {
                         value += 500;
                     }
-                    chance = value / 15; // 33% for every 500 experience
+
+                    return value / 15; // 33% for every 500 experience
+                }();
+
+                if ( Rand::Get( 1, 100 ) > chanceToChooseExp ) {
+                    return { gold, {}, {} };
                 }
 
-                if ( chance ) {
-                    const uint32_t randomRoll = Rand::Get( 1, 100 );
-                    if ( randomRoll <= chance ) {
-                        gold = 0;
-                        hero.IncreaseExperience( experience );
-                    }
-                }
+                return { std::optional<uint32_t>{}, exp, {} };
             }
-            else if ( art.isValid() && !hero.PickupArtifact( art ) ) {
-                gold = GoldInsteadArtifact( objectType );
+
+            if ( !isArtValid ) {
+                return {};
             }
+
+            return { std::optional<uint32_t>{}, {}, art };
+        }();
+
+        if ( goldReward ) {
+            assert( goldReward > 0U );
+
+            hero.GetKingdom().AddFundsResource( Funds( Resource::GOLD, *goldReward ) );
         }
 
-        if ( gold ) {
-            kingdom.AddFundsResource( Funds( Resource::GOLD, gold ) );
+        if ( experienceReward ) {
+            assert( experienceReward > 0U );
+
+            hero.IncreaseExperience( *experienceReward );
+        }
+
+        if ( artifactReward ) {
+            assert( artifactReward->isValid() );
+
+            if ( !hero.PickupArtifact( *artifactReward ) ) {
+                assert( 0 );
+            }
         }
 
         removeMainObjectFromTile( tile );
@@ -735,6 +807,8 @@ namespace
                 removeObjectProtection();
 
                 setColorOnTile( tile, hero.GetColor() );
+
+                castGuardianSpellOnCapturedObject( hero );
             };
 
             if ( isCaptureObjectProtected( tile ) ) {
@@ -805,7 +879,7 @@ namespace
             resetObjectMetadata( tile );
         }
 
-        hero.SetVisitedWideTile( dst_index, objectType, Visit::GLOBAL );
+        hero.SetVisited( dst_index, Visit::GLOBAL );
     }
 
     void AIToWagon( Heroes & hero, int32_t dst_index )
@@ -979,8 +1053,6 @@ namespace
     {
         DEBUG_LOG( DBG_AI, DBG_INFO, hero.GetName() << ", object: " << MP2::StringObject( objectType ) )
 
-        const Maps::Tile & tile = world.getTile( dst_index );
-
         int skill = Skill::Primary::UNKNOWN;
 
         switch ( objectType ) {
@@ -1000,16 +1072,15 @@ namespace
             skill = AISelectSkillFromArena( hero );
             break;
         default:
+            // Did you add a new primary skill object? Add the logic for it!
+            assert( 0 );
             break;
         }
 
-        if ( ( MP2::OBJ_ARENA == objectType && !hero.isObjectTypeVisited( objectType ) ) || !hero.isVisited( tile ) ) {
+        if ( ( MP2::OBJ_ARENA == objectType && !hero.isObjectTypeVisited( objectType ) ) || !hero.isVisited( world.getTile( dst_index ) ) ) {
             // increase skill
             hero.IncreasePrimarySkill( skill );
             hero.SetVisited( dst_index );
-
-            // fix double action tile
-            hero.SetVisitedWideTile( dst_index, objectType );
         }
     }
 
@@ -1026,11 +1097,13 @@ namespace
             exp = 1000;
             break;
         default:
+            // Did you add a new experience object? Add the logic for it!
+            assert( 0 );
             break;
         }
 
         // check already visited
-        if ( !hero.isVisited( tile ) && exp ) {
+        if ( !hero.isVisited( tile ) ) {
             hero.SetVisited( dst_index );
             hero.IncreaseExperience( exp );
         }
@@ -1057,6 +1130,10 @@ namespace
 
         // It is important to mark it globally so other heroes will know about the object.
         hero.SetVisited( dst_index, Visit::GLOBAL );
+
+        // AI does not know about Witch's Hut skill before visiting it.
+        // Share this information with allies.
+        AI::shareObjectVisitInfoWithAllies( hero.GetKingdom(), dst_index );
     }
 
     void AIToShrine( Heroes & hero, int32_t dst_index )
@@ -1074,6 +1151,10 @@ namespace
 
         // All heroes will know which spell is here.
         hero.SetVisited( dst_index, Visit::GLOBAL );
+
+        // AI does not know about Shrine's spell before visiting it.
+        // Share this information with allies.
+        AI::shareObjectVisitInfoWithAllies( hero.GetKingdom(), dst_index );
     }
 
     void AIToGoodLuckObject( Heroes & hero, int32_t dst_index )
@@ -1102,13 +1183,9 @@ namespace
 
         // check already visited
         if ( !hero.isObjectTypeVisited( objectType ) ) {
-            // modify morale
+            // Morale is modified while marking visited object.
             hero.SetVisited( dst_index );
-            if ( move )
-                hero.IncreaseMovePoints( move );
-
-            // fix double action tile
-            hero.SetVisitedWideTile( dst_index, objectType );
+            hero.IncreaseMovePoints( move );
         }
     }
 
@@ -1135,7 +1212,7 @@ namespace
             hero.SetSpellPoints( max * 2 );
         }
 
-        hero.SetVisitedWideTile( dst_index, objectType, Visit::GLOBAL );
+        hero.SetVisited( dst_index, Visit::GLOBAL );
     }
 
     void AIToXanadu( Heroes & hero, int32_t dst_index )
@@ -1297,8 +1374,9 @@ namespace
         else if ( 0 == gold && !hero.isObjectTypeVisited( objectType ) ) {
             // Modify morale
             hero.SetVisited( dst_index );
-            hero.SetVisited( dst_index, Visit::GLOBAL );
         }
+
+        hero.SetVisited( dst_index, Visit::GLOBAL );
     }
 
     void AIToPyramid( Heroes & hero, int32_t dst_index )
@@ -1341,8 +1419,9 @@ namespace
         }
         else {
             hero.SetVisited( dst_index, Visit::LOCAL );
-            hero.SetVisited( dst_index, Visit::GLOBAL );
         }
+
+        hero.SetVisited( dst_index, Visit::GLOBAL );
     }
 
     void AIToObelisk( Heroes & hero, const Maps::Tile & tile )
@@ -1393,6 +1472,7 @@ namespace
         }
 
         Maps::Tile & tile = world.getTile( dst_index );
+        Kingdom & kingdom = hero.GetKingdom();
 
         if ( doesTileContainValuableItems( tile ) ) {
             Army army( tile );
@@ -1409,6 +1489,10 @@ namespace
 
             resetObjectMetadata( tile );
         }
+
+        // Even if the hero has been defeated by a demon (and no longer belongs to any
+        // valid kingdom), this tile should be marked as visited for his former kingdom.
+        kingdom.SetVisited( dst_index, tile.getMainObjectType( false ) );
     }
 
     void AIToDwellingJoinMonster( Heroes & hero, int32_t dst_index )
@@ -1432,6 +1516,8 @@ namespace
         }
 
         setMonsterCountOnTile( tile, 0 );
+
+        hero.SetVisited( dst_index, Visit::GLOBAL );
     }
 
     void AIToDwellingRecruitMonster( Heroes & hero, const MP2::MapObjectType objectType, int32_t dst_index )
@@ -1478,6 +1564,9 @@ namespace
             removeMainObjectFromTile( tile );
             resetObjectMetadata( tile );
         }
+        else {
+            hero.SetVisited( dst_index, Visit::GLOBAL );
+        }
     }
 
     void AIToDwellingBattleMonster( Heroes & hero, const MP2::MapObjectType objectType, const int32_t tileIndex )
@@ -1492,10 +1581,10 @@ namespace
             return;
         }
 
-        Maps::Tile & tile = world.getTile( tileIndex );
+        const Maps::Tile & tile = world.getTile( tileIndex );
         bool recruitmentAllowed = true;
 
-        if ( getColorFromTile( tile ) == Color::NONE ) {
+        if ( getColorFromTile( tile ) == PlayerColor::NONE ) {
             Army army( tile );
 
             const Battle::Result res = Battle::Loader( hero.GetArmy(), army, tileIndex );
@@ -1503,7 +1592,7 @@ namespace
                 hero.IncreaseExperience( res.GetExperienceAttacker() );
 
                 // Set ownership of the dwelling to a Neutral (gray) player so that any player can recruit troops without a fight.
-                setColorOnTile( tile, Color::UNUSED );
+                setColorOnTile( tile, PlayerColor::UNUSED );
             }
             else {
                 AIBattleLose( hero, res, true );
@@ -1561,6 +1650,8 @@ namespace
             Maps::restoreAbandonedMine( tile, Resource::GOLD );
             hero.setObjectTypeUnderHero( MP2::OBJ_MINE );
             setColorOnTile( tile, hero.GetColor() );
+
+            castGuardianSpellOnCapturedObject( hero );
         }
         else {
             AIBattleLose( hero, result, true );
@@ -1574,7 +1665,7 @@ namespace
         Maps::Tile & tile = world.getTile( dst_index );
         const Kingdom & kingdom = hero.GetKingdom();
 
-        if ( kingdom.IsVisitTravelersTent( getColorFromTile( tile ) ) ) {
+        if ( kingdom.IsVisitTravelersTent( getBarrierColorFromTile( tile ) ) ) {
             removeMainObjectFromTile( tile );
             resetObjectMetadata( tile );
         }
@@ -1587,7 +1678,7 @@ namespace
         const Maps::Tile & tile = world.getTile( dst_index );
         Kingdom & kingdom = hero.GetKingdom();
 
-        kingdom.SetVisitTravelersTent( getColorFromTile( tile ) );
+        kingdom.SetVisitTravelersTent( getBarrierColorFromTile( tile ) );
     }
 
     void AIToShipwreckSurvivor( Heroes & hero, const MP2::MapObjectType objectType, int32_t dst_index )
@@ -1634,8 +1725,9 @@ namespace
             }
         }
         else if ( condition == Maps::ArtifactCaptureCondition::HAVE_WISDOM_SKILL || condition == Maps::ArtifactCaptureCondition::HAVE_LEADERSHIP_SKILL ) {
-            // TODO: do we need to check this condition?
-            result = true;
+            const Skill::Secondary & skill = getArtifactSecondarySkillRequirement( tile );
+
+            result = hero.HasSecondarySkill( skill.Skill() );
         }
         else if ( condition >= Maps::ArtifactCaptureCondition::FIGHT_50_ROGUES && condition <= Maps::ArtifactCaptureCondition::FIGHT_1_BONE_DRAGON ) {
             Army army( tile );
@@ -1762,7 +1854,7 @@ namespace
             hero.SetVisited( tileIndex, Visit::GLOBAL );
 
             const auto & eyeMagiIndexes = world.getAllEyeOfMagiPositions();
-            const uint32_t distance = GameStatic::getFogDiscoveryDistance( GameStatic::FogDiscoveryType::MAGI_EYES );
+            const int32_t distance = GameStatic::getFogDiscoveryDistance( GameStatic::FogDiscoveryType::MAGI_EYES );
             for ( const int32_t index : eyeMagiIndexes ) {
                 Maps::ClearFog( index, distance, hero.GetColor() );
             }
@@ -1790,9 +1882,9 @@ namespace
                     artifact = Artifact::UNKNOWN;
                 }
             }
-        }
 
-        DEBUG_LOG( DBG_AI, DBG_INFO, hero.GetName() << " removed " << cursed << " artifacts" )
+            DEBUG_LOG( DBG_AI, DBG_INFO, hero.GetName() << " removed " << cursed << " artifacts" )
+        }
     }
 
     void AIToSirens( Heroes & hero, const MP2::MapObjectType objectType, const int32_t objectIndex )
@@ -1819,6 +1911,15 @@ namespace
         (void)hero;
 #endif
     }
+
+    void AIToBlackCatObject( Heroes & hero, int32_t dst_index )
+    {
+        DEBUG_LOG( DBG_AI, DBG_INFO, hero.GetName() )
+
+        if ( !hero.isObjectTypeVisited( MP2::OBJ_BLACK_CAT ) ) {
+            hero.SetVisited( dst_index );
+        }
+    }
 }
 
 void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
@@ -1829,9 +1930,15 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
     const MP2::MapObjectType objectType = tile.getMainObjectType( dst_index != hero.GetIndex() );
 
     const bool isActionObject = MP2::isInGameActionObject( objectType, hero.isShipMaster() );
-    if ( isActionObject )
+    if ( isActionObject ) {
         hero.SetModes( Heroes::ACTION );
 
+        if ( AIIsShowAnimationForHero( hero, AIGetAllianceColors() ) ) {
+            // Most likely there will be some action, center the map on the hero to avoid subsequent minor screen movements.
+            Interface::AdventureMap::Get().getGameArea().SetCenter( hero.GetCenter() );
+        }
+    }
+
     switch ( objectType ) {
     case MP2::OBJ_BOAT:
         AIToBoat( hero, dst_index );
@@ -1851,7 +1958,7 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
         AIToCastle( hero, dst_index );
         break;
 
-    // pickup object
+    case MP2::OBJ_BARREL:
     case MP2::OBJ_BOTTLE:
     case MP2::OBJ_CAMPFIRE:
     case MP2::OBJ_RESOURCE:
@@ -1898,7 +2005,6 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
         AIToShipwreckSurvivor( hero, objectType, dst_index );
         break;
 
-    // event
     case MP2::OBJ_EVENT:
         AIToEvent( hero, dst_index );
         break;
@@ -1907,7 +2013,6 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
         AIToSign( hero, dst_index );
         break;
 
-    // increase view
     case MP2::OBJ_OBSERVATION_TOWER:
         AIToObservationTower( hero, dst_index );
         break;
@@ -1915,7 +2020,6 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
         AIToMagellanMaps( hero, dst_index );
         break;
 
-    // teleports
     case MP2::OBJ_STONE_LITHS:
         AIToTeleports( hero, dst_index );
         break;
@@ -1923,7 +2027,6 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
         AIToWhirlpools( hero, dst_index );
         break;
 
-    // primary skill modification
     case MP2::OBJ_FORT:
     case MP2::OBJ_MERCENARY_CAMP:
     case MP2::OBJ_WITCH_DOCTORS_HUT:
@@ -1932,24 +2035,20 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
         AIToPrimarySkillObject( hero, objectType, dst_index );
         break;
 
-    // experience modification
     case MP2::OBJ_GAZEBO:
         AIToExperienceObject( hero, objectType, dst_index );
         break;
 
-    // Witch's hut
     case MP2::OBJ_WITCHS_HUT:
         AIToWitchsHut( hero, dst_index );
         break;
 
-    // shrine circle
     case MP2::OBJ_SHRINE_FIRST_CIRCLE:
     case MP2::OBJ_SHRINE_SECOND_CIRCLE:
     case MP2::OBJ_SHRINE_THIRD_CIRCLE:
         AIToShrine( hero, dst_index );
         break;
 
-    // luck modification
     case MP2::OBJ_FOUNTAIN:
     case MP2::OBJ_FAERIE_RING:
     case MP2::OBJ_IDOL:
@@ -1957,7 +2056,6 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
         AIToGoodLuckObject( hero, dst_index );
         break;
 
-    // morale modification
     case MP2::OBJ_OASIS:
     case MP2::OBJ_TEMPLE:
     case MP2::OBJ_WATERING_HOLE:
@@ -1969,7 +2067,6 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
         AIToObelisk( hero, tile );
         break;
 
-    // magic point
     case MP2::OBJ_ARTESIAN_SPRING:
         AIToArtesianSpring( hero, objectType, dst_index );
         break;
@@ -1977,7 +2074,6 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
         AIToMagicWell( hero, dst_index );
         break;
 
-    // increase skill
     case MP2::OBJ_XANADU:
         AIToXanadu( hero, dst_index );
         break;
@@ -2004,7 +2100,6 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
         AIToTreeKnowledge( hero, dst_index );
         break;
 
-    // accept army
     case MP2::OBJ_WATCH_TOWER:
     case MP2::OBJ_EXCAVATION:
     case MP2::OBJ_CAVE:
@@ -2017,13 +2112,12 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
         AIToDwellingJoinMonster( hero, dst_index );
         break;
 
-    // recruit army
     case MP2::OBJ_RUINS:
     case MP2::OBJ_TREE_CITY:
     case MP2::OBJ_WAGON_CAMP:
     case MP2::OBJ_DESERT_TENT:
     case MP2::OBJ_GENIE_LAMP:
-    // loyalty version objects
+    // Price of Loyalty objects
     case MP2::OBJ_WATER_ALTAR:
     case MP2::OBJ_AIR_ALTAR:
     case MP2::OBJ_FIRE_ALTAR:
@@ -2032,7 +2126,6 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
         AIToDwellingRecruitMonster( hero, objectType, dst_index );
         break;
 
-    // recruit army (battle)
     case MP2::OBJ_DRAGON_CITY:
     case MP2::OBJ_CITY_OF_DEAD:
     case MP2::OBJ_TROLL_BRIDGE:
@@ -2073,14 +2166,18 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
         // AI must have some action even if it goes on this object by mistake.
         AIToSirens( hero, objectType, dst_index );
         break;
+    case MP2::OBJ_BLACK_CAT:
+        AIToBlackCatObject( hero, dst_index );
+        break;
     default:
         // AI should know what to do with this type of action object! Please add logic for it.
         assert( !isActionObject );
         break;
     }
 
-    if ( MP2::isNeedStayFront( objectType ) )
+    if ( MP2::isNeedStayFront( objectType ) ) {
         hero.GetPath().Reset();
+    }
 
     // Ignore empty tiles
     if ( isActionObject ) {
@@ -2088,7 +2185,7 @@ void AI::HeroesAction( Heroes & hero, const int32_t dst_index )
     }
 }
 
-void AI::HeroesMove( Heroes & hero )
+fheroes2::GameMode AI::HeroesMove( Heroes & hero )
 {
     const Route::Path & path = hero.GetPath();
 
@@ -2103,11 +2200,11 @@ void AI::HeroesMove( Heroes & hero )
 
         // This is the end of a this hero's movement, even if the hero's full path doesn't end in this town or castle.
         // The further path of this hero will be re-planned by AI.
-        return;
+        return fheroes2::GameMode::END_TURN;
     }
 
     if ( !path.isValidForMovement() ) {
-        return;
+        return fheroes2::GameMode::END_TURN;
     }
 
     hero.SetMove( true );
@@ -2117,7 +2214,7 @@ void AI::HeroesMove( Heroes & hero )
 
     const Settings & conf = Settings::Get();
 
-    const uint32_t colors = AIGetAllianceColors();
+    const PlayerColorsSet colors = AIGetAllianceColors();
     bool recenterNeeded = true;
 
     int heroAnimationFrameCount = 0;
@@ -2147,6 +2244,16 @@ void AI::HeroesMove( Heroes & hero )
             hero.Move( true );
             recenterNeeded = true;
 
+            if ( hero.isAction() ) {
+                hero.ResetAction();
+
+                // Check if the game is over after the hero's action.
+                const fheroes2::GameMode gameState = GameOver::Result::Get().checkGameOver();
+                if ( gameState != fheroes2::GameMode::CANCEL ) {
+                    return gameState;
+                }
+            }
+
             // Render a frame only if there is a need to show one.
             if ( Game::validateAnimationDelay( Game::MAPS_DELAY ) ) {
                 // Update Adventure Map objects' animation.
@@ -2173,14 +2280,17 @@ void AI::HeroesMove( Heroes & hero )
 
                 gameArea.ShiftCenter( { heroAnimationOffset.x * heroMovementSkipValue, heroAnimationOffset.y * heroMovementSkipValue } );
                 gameArea.SetRedraw();
+
                 heroAnimationFrameCount -= heroMovementSkipValue;
                 if ( ( heroAnimationFrameCount & 0x3 ) == 0 ) { // % 4
                     hero.SetSpriteIndex( heroAnimationSpriteId );
 
-                    if ( heroAnimationFrameCount == 0 )
+                    if ( heroAnimationFrameCount == 0 ) {
                         resetHeroSprite = true;
-                    else
+                    }
+                    else {
                         ++heroAnimationSpriteId;
+                    }
                 }
                 const int offsetStep = ( ( 4 - ( heroAnimationFrameCount & 0x3 ) ) & 0x3 ); // % 4
                 hero.SetOffset( { heroAnimationOffset.x * offsetStep, heroAnimationOffset.y * offsetStep } );
@@ -2203,6 +2313,7 @@ void AI::HeroesMove( Heroes & hero )
 
                         heroAnimationOffset = movement;
                         gameArea.ShiftCenter( movement );
+
                         heroAnimationFrameCount = 32 - heroMovementSkipValue;
                         heroAnimationSpriteId = hero.GetSpriteIndex();
                         if ( heroMovementSkipValue < 4 ) {
@@ -2214,6 +2325,16 @@ void AI::HeroesMove( Heroes & hero )
                         }
                     }
                 }
+
+                if ( hero.isAction() ) {
+                    hero.ResetAction();
+
+                    // Check if the game is over after the hero's action.
+                    const fheroes2::GameMode gameState = GameOver::Result::Get().checkGameOver();
+                    if ( gameState != fheroes2::GameMode::CANCEL ) {
+                        return gameState;
+                    }
+                }
             }
 
             if ( Game::validateAnimationDelay( Game::MAPS_DELAY ) ) {
@@ -2233,6 +2354,8 @@ void AI::HeroesMove( Heroes & hero )
     }
 
     hero.SetMove( false );
+
+    return fheroes2::GameMode::END_TURN;
 }
 
 void AI::HeroesCastDimensionDoor( Heroes & hero, const int32_t targetIndex )
@@ -2283,7 +2406,7 @@ int32_t AI::HeroesCastSummonBoat( Heroes & hero, const int32_t boatDestinationIn
 
     hero.SpellCasted( summonBoat );
 
-    const int heroColor = hero.GetColor();
+    const PlayerColor heroColor = hero.GetColor();
 
     Interface::GameArea & gameArea = Interface::AdventureMap::Get().getGameArea();
 
@@ -2312,14 +2435,3 @@ int32_t AI::HeroesCastSummonBoat( Heroes & hero, const int32_t boatDestinationIn
 
     return boatSource;
 }
-
-bool AI::HeroesCastAdventureSpell( Heroes & hero, const Spell & spell )
-{
-    if ( !hero.CanCastSpell( spell ) ) {
-        return false;
-    }
-
-    hero.SpellCasted( spell );
-
-    return true;
-}
