@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2010 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -424,11 +424,11 @@ void Battle::Arena::ApplyAction( Command & cmd )
         ApplyActionSurrender( cmd );
         break;
 
-    case CommandType::AUTO_SWITCH:
-        ApplyActionAutoSwitch( cmd );
+    case CommandType::TOGGLE_AUTO_COMBAT:
+        ApplyActionToggleAutoCombat( cmd );
         break;
-    case CommandType::AUTO_FINISH:
-        ApplyActionAutoFinish( cmd );
+    case CommandType::QUICK_COMBAT:
+        ApplyActionQuickCombat( cmd );
         break;
 
     default:
@@ -791,7 +791,7 @@ void Battle::Arena::ApplyActionMorale( Command & cmd )
 
 void Battle::Arena::ApplyActionRetreat( const Command & /* cmd */ )
 {
-    const int currentColor = GetCurrentColor();
+    const PlayerColor currentColor = GetCurrentColor();
 
     if ( !CanRetreatOpponent( currentColor ) ) {
         ERROR_LOG( "Preconditions were not met" )
@@ -833,7 +833,7 @@ void Battle::Arena::ApplyActionSurrender( const Command & /* cmd */ )
         return true;
     };
 
-    const int currentColor = GetCurrentColor();
+    const PlayerColor currentColor = GetCurrentColor();
 
     if ( _army1->GetColor() == currentColor ) {
         Funds cost;
@@ -1325,9 +1325,9 @@ void Battle::Arena::ApplyActionCatapult( Command & cmd )
     }
 }
 
-void Battle::Arena::ApplyActionAutoSwitch( Command & cmd )
+void Battle::Arena::ApplyActionToggleAutoCombat( Command & cmd )
 {
-    const auto checkParameters = []( const int color ) {
+    const auto checkParameters = []( const PlayerColor color ) {
         const Arena * arena = GetArena();
         assert( arena != nullptr );
 
@@ -1342,11 +1342,11 @@ void Battle::Arena::ApplyActionAutoSwitch( Command & cmd )
         return true;
     };
 
-    const int color = cmd.GetNextValue();
+    const PlayerColor color = static_cast<PlayerColor>( cmd.GetNextValue() );
 
     if ( !checkParameters( color ) ) {
         ERROR_LOG( "Invalid parameters: "
-                   << "color: " << Color::String( color ) << " (" << color << ")" )
+                   << "color: " << Color::String( color ) << " (" << static_cast<int>( color ) << ")" )
 
 #ifdef WITH_DEBUG
         assert( 0 );
@@ -1355,22 +1355,22 @@ void Battle::Arena::ApplyActionAutoSwitch( Command & cmd )
         return;
     }
 
-    _autoBattleColors ^= color;
+    _autoCombatColors ^= color;
 
-    DEBUG_LOG( DBG_BATTLE, DBG_TRACE, "color: " << Color::String( color ) << ", status: " << ( ( _autoBattleColors & color ) ? "on" : "off" ) )
+    DEBUG_LOG( DBG_BATTLE, DBG_TRACE, "color: " << Color::String( color ) << ", status: " << ( ( _autoCombatColors & color ) ? "on" : "off" ) )
 
     if ( _interface ) {
         const Player * player = Players::Get( color );
         assert( player );
 
-        std::string msg = ( _autoBattleColors & color ) ? _( "%{name} has turned on the auto battle" ) : _( "%{name} has turned off the auto battle" );
+        std::string msg = ( _autoCombatColors & color ) ? _( "%{name} has turned on the auto combat" ) : _( "%{name} has turned off the auto combat" );
         StringReplace( msg, "%{name}", player->GetName() );
 
         _interface->setStatus( msg, true );
     }
 }
 
-void Battle::Arena::ApplyActionAutoFinish( const Command & /* cmd */ )
+void Battle::Arena::ApplyActionQuickCombat( const Command & /* cmd */ )
 {
     const int army1Control = GetForce1().GetControl();
     const int army2Control = GetForce2().GetControl();
@@ -1387,14 +1387,14 @@ void Battle::Arena::ApplyActionAutoFinish( const Command & /* cmd */ )
 
     DEBUG_LOG( DBG_BATTLE, DBG_TRACE, "finishing the battle" )
 
-    const int army1Color = GetArmy1Color();
-    const int army2Color = GetArmy2Color();
+    const PlayerColor army1Color = GetArmy1Color();
+    const PlayerColor army2Color = GetArmy2Color();
 
     if ( army1Control & CONTROL_HUMAN ) {
-        _autoBattleColors |= army1Color;
+        _autoCombatColors |= army1Color;
     }
     if ( army2Control & CONTROL_HUMAN ) {
-        _autoBattleColors |= army2Color;
+        _autoCombatColors |= army2Color;
     }
 
     _interface.reset();
@@ -1539,7 +1539,7 @@ void Battle::Arena::ApplyActionSpellTeleport( Command & cmd )
         targetsInfo.push_back( targetInfo );
 
         _interface->RedrawActionSpellCastStatus( Spell( Spell::TELEPORT ), src, commander->GetName(), targetsInfo );
-        _interface->RedrawActionTeleportSpell( *unit, pos.GetHead()->GetIndex() );
+        _interface->redrawActionTeleportSpell( *unit, pos.GetHead()->GetIndex() );
     }
 
     unit->SetPosition( pos );
