@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2010 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -21,8 +21,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
-#ifndef H2BATTLE_INTERFACE_H
-#define H2BATTLE_INTERFACE_H
+#pragma once
 
 #include <cstdint>
 #include <memory>
@@ -34,6 +33,7 @@
 #include "battle_animation.h"
 #include "battle_board.h"
 #include "battle_troop.h"
+#include "color.h"
 #include "cursor.h"
 #include "dialog.h"
 #include "icn.h"
@@ -221,7 +221,7 @@ namespace Battle
 
         TurnOrder & operator=( const TurnOrder & ) = delete;
 
-        void set( const fheroes2::Rect & roi, const std::shared_ptr<const Units> & units, const int opponentColor )
+        void set( const fheroes2::Rect & roi, const std::shared_ptr<const Units> & units, const PlayerColor opponentColor )
         {
             _area = roi;
             _orderOfUnits = units;
@@ -229,7 +229,7 @@ namespace Battle
         }
 
         void redraw( const Unit * current, const uint8_t currentUnitColor, fheroes2::Image & output );
-        void queueEventProcessing( std::string & msg, const fheroes2::Point & offset ) const;
+        bool queueEventProcessing( std::string & msg, const fheroes2::Point & offset ) const;
 
     private:
         enum ArmyColor : uint8_t
@@ -250,7 +250,7 @@ namespace Battle
                           fheroes2::Image & output ) const;
 
         std::weak_ptr<const Units> _orderOfUnits;
-        int _opponentColor{ 0 };
+        PlayerColor _opponentColor{ PlayerColor::NONE };
         fheroes2::Rect _area;
         std::vector<UnitPos> _rects;
     };
@@ -342,7 +342,7 @@ namespace Battle
         void RedrawActionTowerPart2( const Tower & tower, const TargetInfo & target );
         void RedrawActionCatapultPart1( const CastleDefenseStructure catapultTarget, const bool isHit );
         void RedrawActionCatapultPart2( const CastleDefenseStructure catapultTarget );
-        void RedrawActionTeleportSpell( Unit & target, const int32_t dst );
+        void redrawActionTeleportSpell( Unit & target, const int32_t dst );
         void redrawActionEarthquakeSpellPart1( const std::vector<CastleDefenseStructure> & targets );
         void redrawActionEarthquakeSpellPart2( const std::vector<CastleDefenseStructure> & targets );
         void RedrawActionSummonElementalSpell( Unit & target );
@@ -366,8 +366,13 @@ namespace Battle
         void RedrawCover();
         void _redrawBattleGround();
         void _redrawCoverStatic();
-        void RedrawLowObjects( const int32_t cellId );
-        void RedrawHighObjects( const int32_t cellId );
+
+        // Draws cracks and pools that are not higher than the ground level.
+        void _redrawGroundObjects( const int32_t cellId );
+
+        // Draws trees, rocks, bushes and other objects that are higher than the ground level.
+        void _redrawHighObjects( const int32_t cellId );
+
         void RedrawCastle( const Castle & castle, const int32_t cellId );
         void RedrawCastleMainTower( const Castle & castle );
         void RedrawKilled();
@@ -378,16 +383,16 @@ namespace Battle
         void RedrawArmies();
         void RedrawTroopSprite( const Unit & unit );
 
-        fheroes2::Point drawTroopSprite( const Unit & unit, const fheroes2::Sprite & troopSprite );
+        fheroes2::Point _drawTroopSprite( const Unit & unit, const fheroes2::Sprite & troopSprite );
 
         void RedrawTroopCount( const Unit & unit );
 
         void RedrawActionWincesKills( const TargetsInfo & targets, Unit * attacker = nullptr, const Unit * defender = nullptr );
         void RedrawActionArrowSpell( const Unit & target );
         void RedrawActionColdRaySpell( Unit & target );
-        void RedrawActionDisruptingRaySpell( const Unit & target );
-        void RedrawActionBloodLustSpell( const Unit & target );
-        void RedrawActionStoneSpell( const Unit & target );
+        void _redrawActionDisruptingRaySpell( Unit & target );
+        void _redrawActionBloodLustSpell( const Unit & target );
+        void _redrawActionStoneSpell( const Unit & target );
         void RedrawActionColdRingSpell( const int32_t dst, const TargetsInfo & targets );
         void RedrawActionElementalStormSpell( const TargetsInfo & targets );
         void RedrawActionArmageddonSpell();
@@ -415,24 +420,25 @@ namespace Battle
         void SwitchAllUnitsAnimation( const int32_t animationState ) const;
         void UpdateContourColor();
         void CheckGlobalEvents( LocalEvent & );
-        void InterruptAutoBattleIfRequested( LocalEvent & le );
-        void SetHeroAnimationReactionToTroopDeath( const int32_t deathColor ) const;
+        void InterruptAutoCombatIfRequested( LocalEvent & le );
+        void SetHeroAnimationReactionToTroopDeath( const PlayerColor deathColor ) const;
 
         void ProcessingHeroDialogResult( const int result, Actions & actions );
 
         void _openBattleSettingsDialog();
-        void EventStartAutoBattle( const Unit & unit, Actions & actions );
-        void EventAutoFinish( Actions & actions );
+        bool EventStartAutoCombat( const Unit & unit, Actions & actions );
+        bool EventQuickCombat( Actions & actions );
+        void OpenAutoModeDialog( const Unit & unit, Actions & actions );
         void EventShowOptions();
-        void ButtonAutoAction( const Unit & unit, Actions & actions );
-        void ButtonSettingsAction();
-        void ButtonSkipAction( Actions & actions );
         void MouseLeftClickBoardAction( const int themes, const Cell & cell, const bool isConfirmed, Actions & actions );
-        void MousePressRightBoardAction( const Cell & cell ) const;
+        bool MousePressRightBoardAction( const Cell & cell ) const;
 
         int GetBattleCursor( std::string & statusMsg ) const;
         int GetBattleSpellCursor( std::string & statusMsg ) const;
 
+        void _startAutoCombat( const Unit & unit, Actions & actions );
+        void _quickCombat( Actions & actions );
+
         Arena & arena;
         Dialog::FrameBorder border;
 
@@ -448,9 +454,9 @@ namespace Battle
         int _battleGroundIcn{ ICN::UNKNOWN };
         int _borderObjectsIcn{ ICN::UNKNOWN };
 
-        fheroes2::Button btn_auto;
-        fheroes2::Button btn_settings;
-        fheroes2::Button btn_skip;
+        fheroes2::Button _buttonAuto;
+        fheroes2::Button _buttonSettings;
+        fheroes2::Button _buttonSkip;
         Status status;
 
         std::unique_ptr<OpponentSprite> _opponent1;
@@ -462,7 +468,7 @@ namespace Battle
         uint32_t animation_flags_frame{ 0 };
         int catapult_frame{ 0 };
 
-        int _interruptAutoBattleForColor{ 0 };
+        PlayerColor _interruptAutoCombatForColor{ PlayerColor::NONE };
 
         // The Channel ID of pre-battle sound. Used to check it is over to start the battle music.
         std::optional<int> _preBattleSoundChannelId{ -1 };
@@ -476,11 +482,11 @@ namespace Battle
         const Unit * _currentUnit{ nullptr };
         const Unit * _movingUnit{ nullptr };
         const Unit * _flyingUnit{ nullptr };
-        const fheroes2::Sprite * b_current_sprite{ nullptr };
+        const fheroes2::Sprite * _spriteInsteadCurrentUnit{ nullptr };
         fheroes2::Point _movingPos;
         fheroes2::Point _flyingPos;
 
-        int32_t _curentCellIndex{ -1 };
+        int32_t _currentCellIndex{ -1 };
         // Index of the cell selected as the source for the Teleport spell
         int32_t _teleportSpellSrcIdx{ -1 };
         fheroes2::Rect _ballistaTowerRect;
@@ -619,5 +625,3 @@ namespace Battle
         };
     };
 }
-
-#endif
