@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -74,6 +74,7 @@
 #include "tools.h"
 #include "translations.h"
 #include "ui_dialog.h"
+#include "ui_font.h"
 #include "week.h"
 #include "world.h"
 
@@ -99,13 +100,6 @@ namespace
             gettext_noop( "Sheltemburg" ),  gettext_noop( "Corackston" ) };
 }
 
-Castle::Castle( const int32_t posX, const int32_t posY, int race )
-    : MapPosition( { posX, posY } )
-    , _race( race )
-{
-    // Do nothing.
-}
-
 void Castle::LoadFromMP2( const std::vector<uint8_t> & data )
 {
     assert( data.size() == MP2::MP2_CASTLE_STRUCTURE_SIZE );
@@ -222,25 +216,25 @@ void Castle::LoadFromMP2( const std::vector<uint8_t> & data )
     const uint8_t ownerColor = dataStream.get();
     switch ( ownerColor ) {
     case 0:
-        SetColor( Color::BLUE );
+        SetColor( PlayerColor::BLUE );
         break;
     case 1:
-        SetColor( Color::GREEN );
+        SetColor( PlayerColor::GREEN );
         break;
     case 2:
-        SetColor( Color::RED );
+        SetColor( PlayerColor::RED );
         break;
     case 3:
-        SetColor( Color::YELLOW );
+        SetColor( PlayerColor::YELLOW );
         break;
     case 4:
-        SetColor( Color::ORANGE );
+        SetColor( PlayerColor::ORANGE );
         break;
     case 5:
-        SetColor( Color::PURPLE );
+        SetColor( PlayerColor::PURPLE );
         break;
     default:
-        SetColor( Color::NONE );
+        SetColor( PlayerColor::NONE );
         break;
     }
 
@@ -372,8 +366,8 @@ void Castle::LoadFromMP2( const std::vector<uint8_t> & data )
         _race = Race::NECR;
         break;
     default: {
-        const uint32_t kingdomRace = Players::GetPlayerRace( GetColor() );
-        _race = ( Color::NONE != GetColor() && ( Race::ALL & kingdomRace ) ? kingdomRace : Race::Rand() );
+        const int kingdomRace = Players::GetPlayerRace( GetColor() );
+        _race = ( PlayerColor::NONE != GetColor() && ( Race::ALL & kingdomRace ) ? kingdomRace : Race::Rand() );
         break;
     }
     }
@@ -505,11 +499,10 @@ void Castle::_postLoad()
 
     // MageGuild
     _mageGuild.initialize( _race, HaveLibraryCapability() );
-    // educate heroes and captain
-    _educateHeroes();
+    _trainGuestHeroAndCaptainInMageGuild();
 
     // AI troops auto pack for gray towns
-    if ( Color::NONE == GetColor() && !Modes( CUSTOM_ARMY ) ) {
+    if ( PlayerColor::NONE == GetColor() && !Modes( CUSTOM_ARMY ) ) {
         // towns get 4 reinforcements at the start of the game
         for ( int i = 0; i < 4; ++i )
             _joinRNDArmy();
@@ -598,19 +591,19 @@ bool Castle::isPosition( const fheroes2::Point & pt ) const
     return ( ( pt.x >= mp.x - 1 && pt.x <= mp.x + 1 && ( pt.y == mp.y - 1 || pt.y == mp.y ) ) || ( ( pt.x == mp.x - 2 || pt.x == mp.x + 2 ) && pt.y == mp.y ) );
 }
 
-void Castle::_educateHeroes()
+void Castle::_trainGuestHeroAndCaptainInMageGuild()
 {
-    if ( GetLevelMageGuild() == 0 ) {
+    if ( GetLevelMageGuild() < 1 ) {
         return;
     }
 
     Heroes * hero = world.GetHero( *this );
     if ( hero != nullptr ) {
-        MageGuildEducateHero( *hero );
+        trainHeroInMageGuild( *hero );
     }
 
     if ( _captain.isValid() ) {
-        MageGuildEducateHero( _captain );
+        trainHeroInMageGuild( _captain );
     }
 }
 
@@ -665,6 +658,11 @@ Troops Castle::getAvailableArmy( Funds potentialBudget ) const
     return reinforcement;
 }
 
+void Castle::fixFrenchCharactersInName()
+{
+    fheroes2::fixFrenchCharactersForMP2Map( _name );
+}
+
 double Castle::getArmyRecruitmentValue() const
 {
     return getAvailableArmy( GetKingdom().GetFunds() ).GetStrength();
@@ -831,8 +829,6 @@ uint32_t * Castle::_getDwelling( const uint32_t buildingType )
 
 void Castle::ActionNewDay()
 {
-    _educateHeroes();
-
     SetModes( ALLOW_TO_BUILD_TODAY );
 }
 
@@ -847,7 +843,7 @@ void Castle::ActionNewWeek()
         = { DWELLING_MONSTER1, DWELLING_MONSTER2, DWELLING_MONSTER3, DWELLING_MONSTER4, DWELLING_MONSTER5, DWELLING_MONSTER6,
             DWELLING_UPGRADE2, DWELLING_UPGRADE3, DWELLING_UPGRADE4, DWELLING_UPGRADE5, DWELLING_UPGRADE6, DWELLING_UPGRADE7 };
 
-    const bool isNeutral = ( GetColor() == Color::NONE );
+    const bool isNeutral = ( GetColor() == PlayerColor::NONE );
     const bool isPlagueWeek = ( world.GetWeekType().GetType() == WeekName::PLAGUE );
     const bool isMonsterWeek = ( world.GetWeekType().GetType() == WeekName::MONSTERS );
 
@@ -946,7 +942,7 @@ void Castle::ActionNewWeek()
     }
 }
 
-void Castle::ChangeColor( const int newColor )
+void Castle::ChangeColor( const PlayerColor newColor )
 {
     SetColor( newColor );
     _army.SetColor( newColor );
@@ -1015,12 +1011,7 @@ Heroes * Castle::RecruitHero( Heroes * hero )
         return nullptr;
     }
 
-    Kingdom & currentKingdom = GetKingdom();
-    currentKingdom.OddFundsResource( PaymentConditions::RecruitHero() );
-
-    if ( GetLevelMageGuild() ) {
-        MageGuildEducateHero( *hero );
-    }
+    GetKingdom().OddFundsResource( PaymentConditions::RecruitHero() );
 
     DEBUG_LOG( DBG_GAME, DBG_INFO, _name << ", recruit: " << hero->GetName() )
 
@@ -1264,12 +1255,12 @@ BuildingStatus Castle::GetAllBuildingStatus( const Castle & castle )
 
 bool Castle::BuyBuilding( const uint32_t buildingType )
 {
-    if ( !AllowBuyBuilding( buildingType ) )
+    if ( !AllowBuyBuilding( buildingType ) ) {
         return false;
+    }
 
     GetKingdom().OddFundsResource( PaymentConditions::BuyBuilding( _race, buildingType ) );
 
-    // add build
     _constructedBuildings |= buildingType;
 
     switch ( buildingType ) {
@@ -1284,20 +1275,20 @@ bool Castle::BuyBuilding( const uint32_t buildingType )
     case BUILD_MAGEGUILD3:
     case BUILD_MAGEGUILD4:
     case BUILD_MAGEGUILD5:
-        _educateHeroes();
+        _trainGuestHeroAndCaptainInMageGuild();
         break;
 
     case BUILD_CAPTAIN:
         _captain.LoadDefaults( HeroBase::CAPTAIN, _race );
         _captain.SetSpellPoints( _captain.GetMaxSpellPoints() );
-        if ( GetLevelMageGuild() )
-            MageGuildEducateHero( _captain );
+
+        trainHeroInMageGuild( _captain );
         break;
 
     case BUILD_SPEC:
-        // build library
-        if ( HaveLibraryCapability() )
-            _educateHeroes();
+        if ( HaveLibraryCapability() ) {
+            _trainGuestHeroAndCaptainInMageGuild();
+        }
         break;
 
     case DWELLING_MONSTER1:
@@ -1322,7 +1313,6 @@ bool Castle::BuyBuilding( const uint32_t buildingType )
         break;
     }
 
-    // disable day build
     ResetModes( ALLOW_TO_BUILD_TODAY );
 
     DEBUG_LOG( DBG_GAME, DBG_INFO, _name << " build " << GetStringBuilding( buildingType, _race ) )
@@ -2194,7 +2184,7 @@ void Castle::setName( const std::set<std::string, std::less<>> & usedNames )
 int Castle::GetControl() const
 {
     // Neutral castles & towns are always controlled by AI
-    return ( GetColor() & Color::ALL ) ? GetKingdom().GetControl() : CONTROL_AI;
+    return ( Color::allPlayerColors() & GetColor() ) ? GetKingdom().GetControl() : CONTROL_AI;
 }
 
 uint32_t Castle::GetGrownWell()
@@ -2348,12 +2338,12 @@ Castle * AllCastles::Get( const fheroes2::Point & position ) const
     return _castles[iter->second].get();
 }
 
-void AllCastles::Scout( const int colors ) const
+void AllCastles::Scout( const PlayerColorsSet colors ) const
 {
     for ( const Castle * castle : *this ) {
         assert( castle != nullptr );
 
-        if ( !( castle->GetColor() & colors ) ) {
+        if ( !( colors & castle->GetColor() ) ) {
             continue;
         }
 
@@ -2379,15 +2369,6 @@ void AllCastles::NewWeek() const
     } );
 }
 
-void AllCastles::NewMonth() const
-{
-    std::for_each( begin(), end(), []( const Castle * castle ) {
-        assert( castle != nullptr );
-
-        castle->ActionNewMonth();
-    } );
-}
-
 OStreamBase & operator<<( OStreamBase & stream, const Castle & castle )
 {
     const ColorBase & color = castle;
@@ -2531,6 +2512,9 @@ std::string Castle::GetDescriptionBuilding( const uint32_t buildingType ) const
         }
         break;
     }
+    case BUILD_MOAT:
+        StringReplace( res, "%{count}", GameStatic::GetBattleMoatReduceDefense() );
+        break;
 
     case BUILD_SPEC:
     case BUILD_STATUE: {
