@@ -41,7 +41,6 @@
 #include "screen.h"
 #include "settings.h"
 #include "system.h"
-#include "tools.h"
 #include "translations.h"
 #include "ui_button.h"
 #include "ui_dialog.h"
@@ -124,52 +123,60 @@ bool Dialog::SelectCount( std::string header, const int32_t min, const int32_t m
 
     display.render();
 
-    int result = Dialog::ZERO;
-
     const fheroes2::Rect uiRect = uiElement ? fheroes2::Rect{ uiOffset, uiElement->area() } : fheroes2::Rect{};
 
+    int result = Dialog::ZERO;
+    std::string typedValueBuf;
     LocalEvent & le = LocalEvent::Get();
+
     while ( result == Dialog::ZERO && le.HandleEvents() ) {
         bool needRedraw = false;
 
         if ( buttonMax.isVisible() ) {
-            buttonMax.drawOnState( le.isMouseLeftButtonPressedInArea( buttonMax.area() ) );
+            buttonMax.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonMax.area() ) );
         }
 
         if ( buttonMin.isVisible() ) {
-            buttonMin.drawOnState( le.isMouseLeftButtonPressedInArea( buttonMin.area() ) );
+            buttonMin.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonMin.area() ) );
         }
 
-        if ( fheroes2::processIntegerValueTyping( min, max, selectedValue ) ) {
-            valueSelectionElement.setValue( selectedValue );
+        if ( const auto value = fheroes2::processIntegerValueTyping( min, max, typedValueBuf ); value ) {
+            valueSelectionElement.setValue( *value );
+
             needRedraw = true;
         }
         else if ( buttonMax.isVisible() && le.MouseClickLeft( buttonMax.area() ) ) {
             valueSelectionElement.setValue( max );
+            typedValueBuf.clear();
+
             needRedraw = true;
         }
         else if ( buttonMin.isVisible() && le.MouseClickLeft( buttonMin.area() ) ) {
             valueSelectionElement.setValue( min );
+            typedValueBuf.clear();
+
             needRedraw = true;
         }
+        else if ( valueSelectionElement.processEvents() ) {
+            typedValueBuf.clear();
 
-        if ( valueSelectionElement.processEvents() ) {
-            selectedValue = valueSelectionElement.getValue();
             needRedraw = true;
         }
-
-        if ( uiElement && ( le.isMouseLeftButtonReleasedInArea( uiRect ) || le.isMouseRightButtonPressedInArea( uiRect ) ) ) {
+        else if ( uiElement && ( le.isMouseLeftButtonReleasedInArea( uiRect ) || le.isMouseRightButtonPressedInArea( uiRect ) ) ) {
             uiElement->processEvents( uiOffset );
             display.render();
         }
+        else {
+            result = btnGroups.processEvents();
+        }
 
         if ( needRedraw ) {
             const bool redrawMinMax = SwitchMaxMinButtons( buttonMin, buttonMax, valueSelectionElement.getValue(), min );
+
             valueSelectionElement.draw( display );
+
             display.render( redrawMinMax ? interactionElementsRect : selectionBoxArea );
         }
-
-        result = btnGroups.processEvents();
     }
 
     selectedValue = ( result == Dialog::OK ) ? valueSelectionElement.getValue() : 0;
@@ -197,7 +204,7 @@ bool Dialog::inputString( const fheroes2::TextBase & title, const fheroes2::Text
 
     const fheroes2::Sprite & inputArea = fheroes2::AGG::GetICN( ( isEvilInterface ? ICN::BUYBUILD : ICN::BUYBUILE ), 3 );
 
-    const int32_t inputAreaWidth = isMultiLine ? 224 : inputArea.width();
+    const int32_t inputAreaWidth = isMultiLine ? 226 : inputArea.width();
     const int32_t inputAreaHeight = isMultiLine ? 265 : inputArea.height();
 
     const int32_t textboxHeight = body.height( fheroes2::boxAreaWidthPx );
@@ -225,19 +232,15 @@ bool Dialog::inputString( const fheroes2::TextBase & title, const fheroes2::Text
         inputAreaOffset = { 13, 1, -26, -3 };
         fheroes2::Blit( inputArea, display, dst_pt.x, dst_pt.y );
     }
+
     const fheroes2::Rect textInputArea( dst_pt.x + inputAreaOffset.x, dst_pt.y + inputAreaOffset.y, inputAreaWidth + inputAreaOffset.width,
                                         inputAreaHeight + inputAreaOffset.height );
 
-    fheroes2::ImageRestorer textBackground( display, textInputArea.x, textInputArea.y, textInputArea.width, textInputArea.height );
+    // We add extra 4 pixels to the click area width to help setting the cursor at the end of the line if it is fully filled with text characters.
+    const fheroes2::Rect textEditClickArea{ textInputArea.x, textInputArea.y, textInputArea.width + 4, textInputArea.height };
 
-    bool isCursorVisible = true;
-    const fheroes2::FontType fontType( fheroes2::FontType::normalWhite() );
-    fheroes2::Text text( insertCharToString( result, charInsertPos, isCursorVisible ? '_' : '\x7F' ), fontType, textLanguage );
-    text.keepLineTrailingSpaces();
-    if ( !isMultiLine ) {
-        text.fitToOneRow( textInputArea.width );
-    }
-    text.drawInRoi( textInputArea.x, textInputArea.y + 2, textInputArea.width, display, textInputArea );
+    fheroes2::TextInputField textInput( textInputArea, isMultiLine, true, display, textLanguage );
+    textInput.draw( result, static_cast<int32_t>( charInsertPos ) );
 
     const int okayButtonICNID = isEvilInterface ? ICN::UNIFORM_EVIL_OKAY_BUTTON : ICN::UNIFORM_GOOD_OKAY_BUTTON;
 
@@ -275,24 +278,21 @@ bool Dialog::inputString( const fheroes2::TextBase & title, const fheroes2::Text
 
     display.render();
 
-    // We add extra 4 pixels to the click area width to help setting the cursor at the end of the line if it is fully filled with text characters.
-    const fheroes2::Rect textEditClickArea{ textInputArea.x, textInputArea.y, textInputArea.width + 4, textInputArea.height };
-
     LocalEvent & le = LocalEvent::Get();
 
     Game::AnimateResetDelay( Game::DelayType::CURSOR_BLINK_DELAY );
 
     const bool isInGameKeyboardRequired = System::isVirtualKeyboardSupported();
 
-    while ( le.HandleEvents( Game::isDelayNeeded( { Game::DelayType::CURSOR_BLINK_DELAY } ) ) ) {
+    while ( le.HandleEvents() ) {
         bool redraw = false;
 
         if ( buttonOk.isEnabled() ) {
-            buttonOk.drawOnState( le.isMouseLeftButtonPressedInArea( buttonOk.area() ) );
+            buttonOk.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonOk.area() ) );
         }
 
-        buttonCancel.drawOnState( le.isMouseLeftButtonPressedInArea( buttonCancel.area() ) );
-        buttonVirtualKB.drawOnState( le.isMouseLeftButtonPressedInArea( buttonVirtualKB.area() ) );
+        buttonCancel.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonCancel.area() ) );
+        buttonVirtualKB.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonVirtualKB.area() ) );
 
         // In this dialog we input text so we need to use hotkeys that cannot be use in text typing.
         if ( ( !isMultiLine && le.isKeyPressed( fheroes2::Key::KEY_ENTER ) ) || ( buttonOk.isEnabled() && le.MouseClickLeft( buttonOk.area() ) ) ) {
@@ -306,7 +306,8 @@ bool Dialog::inputString( const fheroes2::TextBase & title, const fheroes2::Text
 
         if ( le.MouseClickLeft( buttonVirtualKB.area() ) || ( isInGameKeyboardRequired && le.MouseClickLeft( textEditClickArea ) ) ) {
             if ( textLanguage.has_value() ) {
-                const fheroes2::LanguageSwitcher switcher( textLanguage.value() );
+                // `textLanguage` certainly contains a value so we can simply access it without calling the `.value()` method.
+                const fheroes2::LanguageSwitcher switcher( *textLanguage );
                 fheroes2::openVirtualKeyboard( result, charLimit );
             }
             else {
@@ -331,15 +332,20 @@ bool Dialog::inputString( const fheroes2::TextBase & title, const fheroes2::Text
             redraw = true;
         }
         else if ( le.MouseClickLeft( textEditClickArea ) ) {
+            size_t newPos;
             if ( textLanguage.has_value() ) {
-                const fheroes2::LanguageSwitcher switcher( textLanguage.value() );
-                charInsertPos = fheroes2::getTextInputCursorPosition( text, charInsertPos, le.getMouseCursorPos(), textInputArea );
+                // `textLanguage` certainly contains a value so we can simply access it without calling the `.value()` method.
+                const fheroes2::LanguageSwitcher switcher( *textLanguage );
+                newPos = textInput.getCursorInTextPosition( le.getMouseLeftButtonPressedPos() );
             }
             else {
-                charInsertPos = fheroes2::getTextInputCursorPosition( text, charInsertPos, le.getMouseCursorPos(), textInputArea );
+                newPos = textInput.getCursorInTextPosition( le.getMouseLeftButtonPressedPos() );
             }
 
-            redraw = true;
+            if ( newPos != charInsertPos ) {
+                charInsertPos = newPos;
+                redraw = true;
+            }
         }
         else if ( le.isMouseRightButtonPressedInArea( buttonCancel.area() ) ) {
             fheroes2::showStandardTextMessage( _( "Cancel" ), _( "Exit this menu without doing anything." ), Dialog::ZERO );
@@ -351,12 +357,6 @@ bool Dialog::inputString( const fheroes2::TextBase & title, const fheroes2::Text
             fheroes2::showStandardTextMessage( _( "Open Virtual Keyboard" ), _( "Click to open the Virtual Keyboard dialog." ), Dialog::ZERO );
         }
 
-        // Text input cursor blink.
-        if ( Game::validateAnimationDelay( Game::DelayType::CURSOR_BLINK_DELAY ) ) {
-            isCursorVisible = !isCursorVisible;
-            redraw = true;
-        }
-
         if ( redraw ) {
             bool redrawOkButton = false;
             if ( result.empty() && buttonOk.isEnabled() ) {
@@ -373,15 +373,13 @@ bool Dialog::inputString( const fheroes2::TextBase & title, const fheroes2::Text
                 display.updateNextRenderRoi( buttonOk.area() );
             }
 
-            text.set( insertCharToString( result, charInsertPos, isCursorVisible ? '_' : '\x7F' ), fontType, textLanguage );
+            textInput.draw( result, static_cast<int32_t>( charInsertPos ) );
 
-            if ( !isMultiLine ) {
-                text.fitToOneRow( textInputArea.width );
-            }
-
-            textBackground.restore();
-            text.drawInRoi( textInputArea.x, textInputArea.y + 2, textInputArea.width, display, textInputArea );
-            display.render( textInputArea );
+            display.render( textInput.getOverallArea() );
+        }
+        else if ( textInput.eventProcessing() ) {
+            // Text input blinking cursor render is done when the render of the filename (with cursor) is not planned.
+            display.render( textInput.getCursorArea() );
         }
     }
 
@@ -476,60 +474,69 @@ int Dialog::ArmySplitTroop( const int32_t freeSlots, const int32_t redistributeM
     fheroes2::Button buttonMax( minMaxButtonOffset.x, minMaxButtonOffset.y, isEvilInterface ? ICN::UNIFORM_EVIL_MAX_BUTTON : ICN::UNIFORM_GOOD_MAX_BUTTON, 0, 1 );
     fheroes2::Button buttonMin( minMaxButtonOffset.x, minMaxButtonOffset.y, isEvilInterface ? ICN::UNIFORM_EVIL_MIN_BUTTON : ICN::UNIFORM_GOOD_MIN_BUTTON, 0, 1 );
 
-    const fheroes2::Rect buttonArea( 5, 0, 61, 25 );
     SwitchMaxMinButtons( buttonMin, buttonMax, redistributeCount, redistributeMin );
 
-    LocalEvent & le = LocalEvent::Get();
-
     display.render();
 
-    // message loop
-    int bres = Dialog::ZERO;
-    while ( bres == Dialog::ZERO && le.HandleEvents() ) {
-        bool redraw_count = false;
+    int btnResult = Dialog::ZERO;
+    std::string typedValueBuf;
+    LocalEvent & le = LocalEvent::Get();
+
+    while ( btnResult == Dialog::ZERO && le.HandleEvents() ) {
+        bool needRedraw = false;
 
         if ( buttonMax.isVisible() ) {
-            buttonMax.drawOnState( le.isMouseLeftButtonPressedInArea( buttonMax.area() ) );
+            buttonMax.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonMax.area() ) );
         }
 
         if ( buttonMin.isVisible() ) {
-            buttonMin.drawOnState( le.isMouseLeftButtonPressedInArea( buttonMin.area() ) );
+            buttonMin.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonMin.area() ) );
         }
 
-        if ( fheroes2::processIntegerValueTyping( redistributeMin, redistributeMax, redistributeCount ) ) {
-            valueSelectionElement.setValue( redistributeCount );
-            redraw_count = true;
+        if ( const auto value = fheroes2::processIntegerValueTyping( redistributeMin, redistributeMax, typedValueBuf ); value ) {
+            valueSelectionElement.setValue( *value );
+
+            needRedraw = true;
         }
         else if ( buttonMax.isVisible() && le.MouseClickLeft( buttonMax.area() ) ) {
-            redistributeCount = redistributeMax;
             valueSelectionElement.setValue( redistributeMax );
-            redraw_count = true;
+            typedValueBuf.clear();
+
+            needRedraw = true;
         }
         else if ( buttonMin.isVisible() && le.MouseClickLeft( buttonMin.area() ) ) {
-            redistributeCount = redistributeMin;
             valueSelectionElement.setValue( redistributeMin );
-            redraw_count = true;
+            typedValueBuf.clear();
+
+            needRedraw = true;
         }
         else if ( valueSelectionElement.processEvents() ) {
-            redistributeCount = valueSelectionElement.getValue();
-            redraw_count = true;
+            typedValueBuf.clear();
+
+            needRedraw = true;
+        }
+        else {
+            btnResult = btnGroups.processEvents();
         }
 
         if ( !ssp.empty() ) {
-            for ( std::vector<fheroes2::Rect>::const_iterator it = vrts.begin(); it != vrts.end(); ++it ) {
-                if ( le.MouseClickLeft( *it ) ) {
-                    ssp.setPosition( it->x, it->y );
+            for ( const auto & rt : vrts ) {
+                if ( le.MouseClickLeft( rt ) ) {
+                    ssp.setPosition( rt.x, rt.y );
                     ssp.show();
+
                     display.render( pos );
                 }
             }
         }
 
-        if ( redraw_count ) {
+        if ( needRedraw ) {
             SwitchMaxMinButtons( buttonMin, buttonMax, valueSelectionElement.getValue(), redistributeMin );
+
             if ( !ssp.empty() ) {
                 ssp.hide();
             }
+
             valueSelectionElement.draw( display );
 
             if ( buttonMax.isVisible() ) {
@@ -542,13 +549,11 @@ int Dialog::ArmySplitTroop( const int32_t freeSlots, const int32_t redistributeM
 
             display.render( pos );
         }
-
-        bres = btnGroups.processEvents();
     }
 
     int result = 0;
 
-    if ( bres == Dialog::OK ) {
+    if ( btnResult == Dialog::OK ) {
         redistributeCount = valueSelectionElement.getValue();
 
         if ( !ssp.isHidden() ) {
@@ -557,6 +562,7 @@ int Dialog::ArmySplitTroop( const int32_t freeSlots, const int32_t redistributeM
             for ( int32_t i = 0; i < freeSlots - 1; ++i ) {
                 if ( rt == vrts[i] ) {
                     result = i + 2;
+
                     break;
                 }
             }
