@@ -54,6 +54,7 @@
 #include "m82.h"
 #include "map_format_helper.h"
 #include "map_format_info.h"
+#include "map_object_info.h"
 #include "maps.h"
 #include "maps_fileinfo.h"
 #include "maps_tiles.h"
@@ -73,6 +74,7 @@
 #include "tools.h"
 #include "translations.h"
 #include "ui_dialog.h"
+#include "ui_font.h"
 #include "world.h"
 
 namespace
@@ -99,43 +101,73 @@ namespace
         return { Heroes::LORDKILBURN, Heroes::CELIA };
     }
 
-    int ObjectVisitedModifiersResult( const std::vector<MP2::MapObjectType> & objectTypes, const Heroes & hero, std::string * strs )
+    int getObjectMoraleModifiers( const std::set<MP2::MapObjectType> & objectTypes, std::string * output )
     {
         int result = 0;
 
         for ( const MP2::MapObjectType objectType : objectTypes ) {
-            if ( hero.isObjectTypeVisited( objectType ) ) {
-                result += GameStatic::ObjectVisitedModifiers( objectType );
-
-                if ( strs ) {
-                    switch ( objectType ) {
-                    case MP2::OBJ_GRAVEYARD:
-                    case MP2::OBJ_NON_ACTION_GRAVEYARD:
-                    case MP2::OBJ_SHIPWRECK:
-                    case MP2::OBJ_NON_ACTION_SHIPWRECK:
-                    case MP2::OBJ_DERELICT_SHIP:
-                    case MP2::OBJ_NON_ACTION_DERELICT_SHIP: {
-                        std::string modRobber = _( "shipAndGraveyard|%{object} robber" );
-                        StringReplace( modRobber, "%{object}", MP2::StringObject( objectType ) );
-                        strs->append( modRobber );
-                        break;
-                    }
-                    case MP2::OBJ_PYRAMID:
-                    case MP2::OBJ_NON_ACTION_PYRAMID: {
-                        std::string modRaided = _( "pyramid|%{object} raided" );
-                        StringReplace( modRaided, "%{object}", MP2::StringObject( objectType ) );
-                        strs->append( modRaided );
-                        break;
-                    }
-                    default:
-                        strs->append( MP2::StringObject( objectType ) );
-                        break;
-                    }
-
-                    fheroes2::appendModifierToString( *strs, GameStatic::ObjectVisitedModifiers( objectType ) );
-                    strs->append( "\n" );
-                }
+            const int32_t modifier = GameStatic::getObjectMoraleEffect( objectType );
+            if ( modifier == 0 ) {
+                continue;
             }
+
+            result += modifier;
+
+            if ( output == nullptr ) {
+                continue;
+            }
+
+            switch ( objectType ) {
+            case MP2::OBJ_DERELICT_SHIP:
+            case MP2::OBJ_GRAVEYARD:
+            case MP2::OBJ_SHIPWRECK: {
+                std::string modRobber = _( "shipAndGraveyard|%{object} robber" );
+                StringReplace( modRobber, "%{object}", MP2::StringObject( objectType ) );
+                output->append( modRobber );
+                break;
+            }
+            default:
+                output->append( MP2::StringObject( objectType ) );
+                break;
+            }
+
+            fheroes2::appendModifierToString( *output, modifier );
+            output->append( "\n" );
+        }
+
+        return result;
+    }
+
+    int getObjectLuckModifiers( const std::set<MP2::MapObjectType> & objectTypes, std::string * output )
+    {
+        int result = 0;
+
+        for ( const MP2::MapObjectType objectType : objectTypes ) {
+            const int32_t modifier = GameStatic::getObjectLuckEffect( objectType );
+            if ( modifier == 0 ) {
+                continue;
+            }
+
+            result += modifier;
+
+            if ( output == nullptr ) {
+                continue;
+            }
+
+            switch ( objectType ) {
+            case MP2::OBJ_PYRAMID: {
+                std::string modRaided = _( "pyramid|%{object} raided" );
+                StringReplace( modRaided, "%{object}", MP2::StringObject( objectType ) );
+                output->append( modRaided );
+                break;
+            }
+            default:
+                output->append( MP2::StringObject( objectType ) );
+                break;
+            }
+
+            fheroes2::appendModifierToString( *output, modifier );
+            output->append( "\n" );
         }
 
         return result;
@@ -236,7 +268,7 @@ Heroes::Heroes( const int heroID, const int race, const uint32_t additionalExper
 
 Heroes::Heroes( int heroid, int rc )
     : HeroBase( HeroBase::HEROES, rc )
-    , ColorBase( Color::NONE )
+    , ColorBase( PlayerColor::NONE )
     , experience( GetStartingXp() )
     , secondary_skills( rc )
     , army( this )
@@ -289,7 +321,7 @@ Heroes::Heroes( int heroid, int rc )
     move_point = GetMaxMovePoints();
 }
 
-void Heroes::LoadFromMP2( const int32_t mapIndex, const int colorType, const int raceType, const bool isInJail, const std::vector<uint8_t> & data )
+void Heroes::LoadFromMP2( const int32_t mapIndex, const PlayerColor colorType, const int raceType, const bool isInJail, const std::vector<uint8_t> & data )
 {
     assert( data.size() == MP2::MP2_HEROES_STRUCTURE_SIZE );
 
@@ -833,7 +865,7 @@ const std::string & Heroes::GetName() const
     return name;
 }
 
-int Heroes::GetColor() const
+PlayerColor Heroes::GetColor() const
 {
     return ColorBase::GetColor();
 }
@@ -1032,20 +1064,17 @@ uint32_t Heroes::GetMaxMovePoints( const bool onWater ) const
 
 int Heroes::GetMorale() const
 {
-    return GetMoraleWithModificators( nullptr );
+    return getMoraleWithModifiers( nullptr );
 }
 
-int Heroes::GetMoraleWithModificators( std::string * strs ) const
+int Heroes::getMoraleWithModifiers( std::string * strs ) const
 {
     int result = Morale::NORMAL;
 
     // bonus leadership
-    result += Skill::GetLeadershipModifiers( GetLevelSkill( Skill::Secondary::LEADERSHIP ), strs );
+    result += Skill::getLeadershipModifiers( GetLevelSkill( Skill::Secondary::LEADERSHIP ), strs );
 
-    // object visited
-    const std::vector<MP2::MapObjectType> objectTypes{ MP2::OBJ_BUOY,      MP2::OBJ_OASIS,         MP2::OBJ_WATERING_HOLE, MP2::OBJ_TEMPLE,
-                                                       MP2::OBJ_GRAVEYARD, MP2::OBJ_DERELICT_SHIP, MP2::OBJ_SHIPWRECK };
-    result += ObjectVisitedModifiersResult( objectTypes, *this, strs );
+    result += getObjectMoraleModifiers( getAllVisitedObjectTypes(), strs );
 
     // bonus artifact
     result += GetMoraleModificator( strs );
@@ -1057,9 +1086,11 @@ int Heroes::GetMoraleWithModificators( std::string * strs ) const
             *strs += maxMoraleArtifact.GetName();
             *strs += _( " gives you maximum morale" );
         }
-        result = Morale::BLOOD;
+
+        return Morale::BLOOD;
     }
-    else if ( strs != nullptr && !strs->empty() && strs->back() == '\n' ) {
+
+    if ( strs != nullptr && !strs->empty() && strs->back() == '\n' ) {
         // Remove the possible empty line at the end of the string.
         strs->pop_back();
     }
@@ -1069,19 +1100,18 @@ int Heroes::GetMoraleWithModificators( std::string * strs ) const
 
 int Heroes::GetLuck() const
 {
-    return GetLuckWithModificators( nullptr );
+    return getLuckWithModifiers( nullptr );
 }
 
-int Heroes::GetLuckWithModificators( std::string * strs ) const
+int Heroes::getLuckWithModifiers( std::string * strs ) const
 {
     int result = Luck::NORMAL;
 
     // bonus luck
-    result += Skill::GetLuckModifiers( GetLevelSkill( Skill::Secondary::LUCK ), strs );
+    result += Skill::getLuckModifiers( GetLevelSkill( Skill::Secondary::LUCK ), strs );
 
     // object visited
-    const std::vector<MP2::MapObjectType> objectTypes{ MP2::OBJ_MERMAID, MP2::OBJ_FAERIE_RING, MP2::OBJ_FOUNTAIN, MP2::OBJ_IDOL, MP2::OBJ_PYRAMID };
-    result += ObjectVisitedModifiersResult( objectTypes, *this, strs );
+    result += getObjectLuckModifiers( getAllVisitedObjectTypes(), strs );
 
     // bonus artifact
     result += GetLuckModificator( strs );
@@ -1092,9 +1122,11 @@ int Heroes::GetLuckWithModificators( std::string * strs ) const
             *strs += maxLuckArtifact.GetName();
             *strs += _( " gives you maximum luck" );
         }
-        result = Luck::IRISH;
+
+        return Luck::IRISH;
     }
-    else if ( strs != nullptr && !strs->empty() && strs->back() == '\n' ) {
+
+    if ( strs != nullptr && !strs->empty() && strs->back() == '\n' ) {
         // Remove the possible empty line at the end of the string.
         strs->pop_back();
     }
@@ -1102,9 +1134,9 @@ int Heroes::GetLuckWithModificators( std::string * strs ) const
     return Luck::Normalize( result );
 }
 
-bool Heroes::Recruit( const int col, const fheroes2::Point & pt )
+bool Heroes::Recruit( const PlayerColor col, const fheroes2::Point & pt )
 {
-    if ( GetColor() != Color::NONE ) {
+    if ( GetColor() != PlayerColor::NONE ) {
         DEBUG_LOG( DBG_GAME, DBG_WARN, "hero has already been hired by some kingdom" )
 
         return false;
@@ -1154,11 +1186,10 @@ bool Heroes::Recruit( const Castle & castle )
         return false;
     }
 
-    if ( castle.GetLevelMageGuild() ) {
-        castle.MageGuildEducateHero( *this );
-    }
+    castle.trainHeroInMageGuild( *this );
 
     SetVisited( GetIndex() );
+
     return true;
 }
 
@@ -1180,11 +1211,6 @@ void Heroes::ActionNewWeek()
     visit_object.remove_if( Visit::isWeekLife );
 }
 
-void Heroes::ActionNewMonth()
-{
-    visit_object.remove_if( Visit::isMonthLife );
-}
-
 void Heroes::ActionAfterBattle()
 {
     visit_object.remove_if( Visit::isBattleLife );
@@ -1209,15 +1235,14 @@ void Heroes::ReplenishSpellPoints()
     const uint32_t maxp = GetMaxSpellPoints();
     uint32_t curr = GetSpellPoints();
 
-    // spell points may be doubled in artesian spring, leave as is
+    // Spell points can be doubled in Artesian Spring, leave them as they are
     if ( curr >= maxp ) {
         return;
     }
 
     const Castle * castle = inCastle();
 
-    // in castle?
-    if ( castle && castle->GetLevelMageGuild() ) {
+    if ( castle && castle->GetLevelMageGuild() > 0 ) {
         SetSpellPoints( maxp );
     }
     else {
@@ -1256,7 +1281,7 @@ const Castle * Heroes::inCastle() const
 
 Castle * Heroes::inCastleMutable() const
 {
-    if ( GetColor() == Color::NONE ) {
+    if ( GetColor() == PlayerColor::NONE ) {
         return nullptr;
     }
 
@@ -1285,55 +1310,96 @@ bool Heroes::isObjectTypeVisited( const MP2::MapObjectType objectType, Visit::Ty
     return std::any_of( visit_object.begin(), visit_object.end(), [objectType]( const IndexObject & v ) { return v.isObject( objectType ); } );
 }
 
-void Heroes::SetVisited( int32_t index, Visit::Type type )
+std::set<MP2::MapObjectType> Heroes::getAllVisitedObjectTypes() const
 {
-    const Maps::Tile & tile = world.getTile( index );
+    std::set<MP2::MapObjectType> objectTypes;
+
+    for ( const auto & object : visit_object ) {
+        objectTypes.emplace( object.second );
+    }
+
+    return objectTypes;
+}
+
+void Heroes::SetVisited( const int32_t tileIndex, const Visit::Type type /* = Visit::LOCAL */ )
+{
+    const Maps::Tile & tile = world.getTile( tileIndex );
+
     const MP2::MapObjectType objectType = tile.getMainObjectType( false );
+    if ( !MP2::isOffGameActionObject( objectType ) ) {
+        // Something is wrong as how are going to visit a non-action object?!
+        assert( 0 );
+        return;
+    }
+
+    const uint32_t objectUID = tile.getMainObjectPart()._uid;
 
     if ( Visit::GLOBAL == type ) {
-        GetKingdom().SetVisited( index, objectType );
+        GetKingdom().SetVisited( tileIndex, objectType );
     }
-    else if ( !isVisited( tile ) && MP2::OBJ_NONE != objectType ) {
-        visit_object.emplace_front( index, objectType );
+    else if ( !isVisited( tile ) ) {
+        visit_object.emplace_front( tileIndex, objectType );
+    }
+
+    // An object could be bigger than 1 tile so we need to check all its tiles.
+    constexpr int32_t searchDist = []() constexpr {
+        constexpr int32_t max = std::max( Maps::maxActionObjectDimensions.width, Maps::maxActionObjectDimensions.height );
+        static_assert( max > 0 );
+
+        return max - 1;
+    }();
+
+    for ( const int32_t index : Maps::getAroundIndexes( tileIndex, searchDist ) ) {
+        const Maps::Tile & currentTile = world.getTile( index );
+        if ( currentTile.getMainObjectType( false ) != objectType || currentTile.getMainObjectPart()._uid != objectUID ) {
+            continue;
+        }
+
+        if ( Visit::GLOBAL == type ) {
+            GetKingdom().SetVisited( index, objectType );
+        }
+        else if ( !isVisited( currentTile ) ) {
+            visit_object.emplace_front( index, objectType );
+        }
     }
 }
 
 void Heroes::setVisitedForAllies( const int32_t tileIndex ) const
 {
     const Maps::Tile & tile = world.getTile( tileIndex );
+
     const MP2::MapObjectType objectType = tile.getMainObjectType( false );
+    if ( !MP2::isOffGameActionObject( objectType ) ) {
+        // Something is wrong as how are going to visit a non-action object?!
+        assert( 0 );
+        return;
+    }
+
+    const uint32_t objectUID = tile.getMainObjectPart()._uid;
 
     // Set visited to all allies as well.
-    const Colors friendColors( Players::GetPlayerFriends( GetColor() ) );
-    for ( const int friendColor : friendColors ) {
+    const PlayerColorsVector friendColors( Players::GetPlayerFriends( GetColor() ) );
+    for ( const PlayerColor friendColor : friendColors ) {
         world.GetKingdom( friendColor ).SetVisited( tileIndex, objectType );
     }
-}
 
-void Heroes::SetVisitedWideTile( int32_t index, const MP2::MapObjectType objectType, Visit::Type type )
-{
-    const Maps::Tile & tile = world.getTile( index );
-    const uint32_t uid = tile.getMainObjectPart()._uid;
-    int wide = 0;
+    // An object could be bigger than 1 tile so we need to check all its tiles.
+    constexpr int32_t searchDist = []() constexpr {
+        constexpr int32_t max = std::max( Maps::maxActionObjectDimensions.width, Maps::maxActionObjectDimensions.height );
+        static_assert( max > 0 );
 
-    switch ( objectType ) {
-    case MP2::OBJ_SKELETON:
-    case MP2::OBJ_OASIS:
-    case MP2::OBJ_STANDING_STONES:
-    case MP2::OBJ_ARTESIAN_SPRING:
-        wide = 2;
-        break;
-    case MP2::OBJ_WATERING_HOLE:
-        wide = 4;
-        break;
-    default:
-        break;
-    }
+        return max - 1;
+    }();
 
-    if ( tile.getMainObjectType( false ) == objectType && wide ) {
-        for ( int32_t ii = tile.GetIndex() - ( wide - 1 ); ii <= tile.GetIndex() + ( wide - 1 ); ++ii )
-            if ( Maps::isValidAbsIndex( ii ) && world.getTile( ii ).getMainObjectPart()._uid == uid )
-                SetVisited( ii, type );
+    for ( const int32_t index : Maps::getAroundIndexes( tileIndex, searchDist ) ) {
+        const Maps::Tile & currentTile = world.getTile( index );
+        if ( currentTile.getMainObjectType( false ) != objectType || currentTile.getMainObjectPart()._uid != objectUID ) {
+            continue;
+        }
+
+        for ( const PlayerColor friendColor : friendColors ) {
+            world.GetKingdom( friendColor ).SetVisited( index, objectType );
+        }
     }
 }
 
@@ -1575,9 +1641,9 @@ uint32_t Heroes::getExperienceMaxValue()
     return 2990600;
 }
 
-bool Heroes::BuySpellBook( const Castle * castle )
+bool Heroes::BuySpellBook( const Castle & castle )
 {
-    if ( HaveSpellBook() || Color::NONE == GetColor() ) {
+    if ( HaveSpellBook() || PlayerColor::NONE == GetColor() || castle.GetLevelMageGuild() < 1 ) {
         return false;
     }
 
@@ -1611,9 +1677,7 @@ bool Heroes::BuySpellBook( const Castle * castle )
     if ( SpellBookActivate() ) {
         kingdom.OddFundsResource( payment );
 
-        if ( castle ) {
-            castle->MageGuildEducateHero( *this );
-        }
+        castle.trainHeroInMageGuild( *this );
 
         return true;
     }
@@ -1717,14 +1781,15 @@ int Heroes::GetLevelSkill( int skill ) const
 
 void Heroes::LearnSkill( const Skill::Secondary & skill )
 {
-    if ( skill.isValid() )
+    if ( skill.isValid() ) {
         secondary_skills.AddSkill( skill );
+    }
 }
 
 void Heroes::Scout( const int tileIndex ) const
 {
     // We should not scout for the NONE color player.
-    assert( GetColor() != Color::NONE );
+    assert( GetColor() != PlayerColor::NONE );
 
     Maps::ClearFog( tileIndex, GetScoutingDistance(), GetColor() );
 
@@ -1742,10 +1807,10 @@ void Heroes::Scout( const int tileIndex ) const
 #endif
 }
 
-int Heroes::GetScoutingDistance() const
+int32_t Heroes::GetScoutingDistance() const
 {
-    return static_cast<int>( GetBagArtifacts().getTotalArtifactEffectValue( fheroes2::ArtifactBonusType::AREA_REVEAL_DISTANCE )
-                             + GameStatic::getFogDiscoveryDistance( GameStatic::FogDiscoveryType::HEROES ) + GetSecondarySkillValue( Skill::Secondary::SCOUTING ) );
+    return GetBagArtifacts().getTotalArtifactEffectValue( fheroes2::ArtifactBonusType::AREA_REVEAL_DISTANCE )
+           + GameStatic::getFogDiscoveryDistance( GameStatic::FogDiscoveryType::HEROES ) + static_cast<int32_t>( GetSecondarySkillValue( Skill::Secondary::SCOUTING ) );
 }
 
 fheroes2::Rect Heroes::GetScoutRoi() const
@@ -1884,7 +1949,7 @@ void Heroes::LevelUpSecondarySkill( const HeroSeedsForLevelUp & seeds, int prima
 
         // Campaign-only heroes get additional experience immediately upon their creation, even while still neutral.
         // We should not try to scout the area around such heroes.
-        if ( selected.Skill() == Skill::Secondary::SCOUTING && GetColor() != Color::NONE ) {
+        if ( selected.Skill() == Skill::Secondary::SCOUTING && GetColor() != PlayerColor::NONE ) {
             Scout( GetIndex() );
             if ( isControlHuman() ) {
                 ScoutRadar();
@@ -1895,10 +1960,12 @@ void Heroes::LevelUpSecondarySkill( const HeroSeedsForLevelUp & seeds, int prima
 
 void Heroes::ApplyPenaltyMovement( uint32_t penalty )
 {
-    if ( move_point >= penalty )
+    if ( move_point >= penalty ) {
         move_point -= penalty;
-    else
+    }
+    else {
         move_point = 0;
+    }
 }
 
 bool Heroes::MayStillMove( const bool ignorePath, const bool ignoreSleeper ) const
@@ -1920,7 +1987,7 @@ bool Heroes::MayStillMove( const bool ignorePath, const bool ignoreSleeper ) con
 
 bool Heroes::MayCastAdventureSpells() const
 {
-    return isValid() && GetColor() != Color::NONE;
+    return isValid() && GetColor() != PlayerColor::NONE;
 }
 
 bool Heroes::isValid() const
@@ -1930,12 +1997,12 @@ bool Heroes::isValid() const
 
 bool Heroes::isActive() const
 {
-    return isValid() && ( GetColor() & Color::ALL ) && !Modes( JAIL );
+    return isValid() && ( Color::allPlayerColors() & GetColor() ) && !Modes( JAIL );
 }
 
 bool Heroes::isAvailableForHire() const
 {
-    return isValid() && GetColor() == Color::NONE && !Modes( JAIL );
+    return isValid() && GetColor() == PlayerColor::NONE && !Modes( JAIL );
 }
 
 void Heroes::Dismiss( int reason )
@@ -1944,18 +2011,19 @@ void Heroes::Dismiss( int reason )
         return;
     }
 
-    // if not surrendering, reset army
+    // Reset army to default state for hero's race if hero did not surrender in battle.
     if ( ( reason & Battle::RESULT_SURRENDER ) == 0 ) {
         army.Reset( true );
     }
 
-    const int heroColor = GetColor();
+    const PlayerColor heroColor = GetColor();
     Kingdom & kingdom = GetKingdom();
 
-    if ( heroColor != Color::NONE ) {
+    if ( heroColor != PlayerColor::NONE ) {
         kingdom.RemoveHero( this );
     }
-    SetColor( Color::NONE );
+
+    SetColor( PlayerColor::NONE );
 
     world.getTile( GetIndex() ).setHero( nullptr );
     SetIndex( -1 );
@@ -1972,7 +2040,7 @@ void Heroes::Dismiss( int reason )
     if ( ( Battle::RESULT_RETREAT | Battle::RESULT_SURRENDER ) & reason ) {
         SetModes( SAVEMP );
 
-        if ( heroColor != Color::NONE ) {
+        if ( heroColor != PlayerColor::NONE ) {
             kingdom.appendSurrenderedHero( *this );
         }
     }
@@ -2133,6 +2201,7 @@ std::string Heroes::String() const
        << "experience      : " << experience << std::endl
        << "level           : " << GetLevel() << std::endl
        << "magic points    : " << GetSpellPoints() << " / " << GetMaxSpellPoints() << std::endl
+       << "morale, luck    : " << GetMorale() << ", " << GetLuck() << std ::endl
        << "position x, y   : " << GetCenter().x << ", " << GetCenter().y << std::endl
        << "move points     : " << move_point << " / " << GetMaxMovePoints() << std::endl
        << "direction       : " << Direction::String( direction ) << std::endl
@@ -2400,12 +2469,12 @@ Heroes * AllHeroes::Get( const fheroes2::Point & center ) const
     return nullptr;
 }
 
-void AllHeroes::Scout( int colors ) const
+void AllHeroes::Scout( PlayerColorsSet colors ) const
 {
     for ( const Heroes * hero : *this ) {
         assert( hero != nullptr );
 
-        if ( !( hero->GetColor() & colors ) ) {
+        if ( ( colors & hero->GetColor() ) == 0 ) {
             continue;
         }
 
@@ -2440,15 +2509,6 @@ void AllHeroes::NewWeek() const
     } );
 }
 
-void AllHeroes::NewMonth() const
-{
-    std::for_each( begin(), end(), []( Heroes * hero ) {
-        assert( hero != nullptr );
-
-        hero->ActionNewMonth();
-    } );
-}
-
 Heroes * AllHeroes::FromJail( int32_t index ) const
 {
     for ( Heroes * hero : *this ) {
@@ -2462,6 +2522,11 @@ Heroes * AllHeroes::FromJail( int32_t index ) const
     return nullptr;
 }
 
+void Heroes::fixFrenchCharactersInName()
+{
+    fheroes2::fixFrenchCharactersForMP2Map( name );
+}
+
 OStreamBase & operator<<( OStreamBase & stream, const VecHeroes & heroes )
 {
     stream.put32( static_cast<uint32_t>( heroes.size() ) );
