@@ -409,6 +409,8 @@ namespace MP2
         // Otherwise, name it with prefix OBJ_.
         OBJ_SWAMPY_LAKE = 257,
         OBJ_FROZEN_LAKE = 258,
+        OBJ_NON_ACTION_BLACK_CAT = 259,
+        OBJ_NON_ACTION_BARREL = 260,
 
         // This section defines all types of action objects which are not present in the original game.
         // If the object by nature is an action object name it with prefix OBJ_.
@@ -416,6 +418,8 @@ namespace MP2
         // The value of the object must be: non-action object value + OBJ_ACTION_OBJECT_TYPE.
         OBJ_ACTION_SWAMPY_LAKE = OBJ_SWAMPY_LAKE + OBJ_ACTION_OBJECT_TYPE,
         OBJ_ACTION_FROZEN_LAKE = OBJ_FROZEN_LAKE + OBJ_ACTION_OBJECT_TYPE,
+        OBJ_BLACK_CAT = OBJ_NON_ACTION_BLACK_CAT + OBJ_ACTION_OBJECT_TYPE,
+        OBJ_BARREL = OBJ_NON_ACTION_BARREL + OBJ_ACTION_OBJECT_TYPE,
     };
 
     enum ObjectIcnType : uint8_t
@@ -530,7 +534,6 @@ namespace MP2
 
     bool isDayLife( const MapObjectType objectType );
     bool isWeekLife( const MapObjectType objectType );
-    bool isMonthLife( const MapObjectType objectType );
     bool isBattleLife( const MapObjectType objectType );
 
     // Make sure that you pass a valid action object.
