@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2020 - 2024                                             *
+ *   Copyright (C) 2020 - 2025                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -18,21 +18,23 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
+#include "world_regions.h"
+
 #include <algorithm>
+#include <cassert>
 #include <cstddef>
 #include <cstdint>
-#include <memory>
 #include <set>
 #include <utility>
 #include <vector>
 
 #include "castle.h"
+#include "ground.h"
 #include "maps.h"
 #include "maps_tiles.h"
 #include "math_base.h"
 #include "mp2.h"
-#include "world.h"
-#include "world_regions.h"
+#include "world.h" // IWYU pragma: associated
 
 namespace
 {
@@ -181,7 +183,7 @@ void World::ComputeStaticAnalysis()
     const uint32_t emptyLineFrequency = 7;
 
     // Reset the region information for all tiles
-    std::for_each( vec_tiles.begin(), vec_tiles.end(), []( Maps::Tiles & tile ) { tile.UpdateRegion( REGION_NODE_BLOCKED ); } );
+    std::for_each( vec_tiles.begin(), vec_tiles.end(), []( Maps::Tile & tile ) { tile.UpdateRegion( REGION_NODE_BLOCKED ); } );
 
     // Step 1. Split map into terrain, water and ground points
     // Initialize the obstacles vector
@@ -200,25 +202,32 @@ void World::ComputeStaticAnalysis()
         obstacles[3].emplace_back( y, 0 ); // ground, rows
     }
 
+    int obstacleCount = 0;
+    int waterCount = 0;
+    uint32_t terrainPenalty = 0;
+
     // Find the terrain
     for ( int y = 0; y < height; ++y ) {
         const int rowIndex = y * width;
         for ( int x = 0; x < width; ++x ) {
             const int index = rowIndex + x;
-            const Maps::Tiles & tile = vec_tiles[index];
+            const Maps::Tile & tile = vec_tiles[index];
             // If tile is blocked (mountain, trees, etc) then it's applied to both
             if ( tile.GetPassable() == 0 ) {
+                ++obstacleCount;
                 ++obstacles[0][x].second;
                 ++obstacles[1][y].second;
                 ++obstacles[2][x].second;
                 ++obstacles[3][y].second;
             }
             else if ( tile.isWater() ) {
+                ++waterCount;
                 // if it's water then ground tiles consider it an obstacle
                 ++obstacles[2][x].second;
                 ++obstacles[3][y].second;
             }
             else {
+                terrainPenalty += Maps::Ground::GetPenalty( tile, 0 );
                 // else then ground is an obstacle for water navigation
                 ++obstacles[0][x].second;
                 ++obstacles[1][y].second;
@@ -226,6 +235,16 @@ void World::ComputeStaticAnalysis()
         }
     }
 
+    const int passableTileCount = ( width * height ) - obstacleCount;
+    assert( passableTileCount > 0 );
+
+    _waterPercentage = static_cast<uint8_t>( waterCount * 100 / passableTileCount );
+
+    const int landTiles = passableTileCount - waterCount;
+    assert( landTiles > 0 );
+
+    _landRoughness = static_cast<double>( terrainPenalty ) / ( landTiles * Maps::Ground::defaultGroundPenalty );
+
     // sort the map rows and columns based on amount of obstacles
     for ( int i = 0; i < 4; ++i )
         std::sort( obstacles[i].begin(), obstacles[i].end(), []( const TileData & left, const TileData & right ) { return left.second < right.second; } );
@@ -244,13 +263,14 @@ void World::ComputeStaticAnalysis()
     std::vector<int> regionCenters;
     TileDataVector castleCenters;
     for ( const Castle * castle : vec_castles ) {
-        castleCenters.emplace_back( castle->GetIndex(), castle->GetColor() );
+        castleCenters.emplace_back( castle->GetIndex(), static_cast<int>( castle->GetColor() ) );
     }
     std::sort( castleCenters.begin(), castleCenters.end(), []( const TileData & left, const TileData & right ) {
         // Sort castles by color primarily (NONE is last)
         // If same color then compare map index
-        if ( left.second == right.second )
+        if ( left.second == right.second ) {
             return left.first < right.first;
+        }
         return left.second > right.second;
     } );
 
@@ -271,7 +291,7 @@ void World::ComputeStaticAnalysis()
                 int centerIndex = -1;
 
                 const int tileIndex = rowIndex + colID;
-                const Maps::Tiles & tile = vec_tiles[tileIndex];
+                const Maps::Tile & tile = vec_tiles[tileIndex];
                 if ( tile.GetPassable() && tile.isWater() ) {
                     centerIndex = tileIndex;
                 }
@@ -279,7 +299,7 @@ void World::ComputeStaticAnalysis()
                     for ( uint8_t direction = 0; direction < 8; ++direction ) {
                         const int newIndex = tileIndex + directionOffsets[direction];
                         if ( newIndex >= 0 && static_cast<size_t>( newIndex ) < totalMapTiles ) {
-                            const Maps::Tiles & newTile = vec_tiles[newIndex];
+                            const Maps::Tile & newTile = vec_tiles[newIndex];
                             if ( newTile.GetPassable() != 0 && tile.isWater() == ( waterOrGround != 0 ) ) {
                                 centerIndex = newIndex;
                                 break;
@@ -302,14 +322,14 @@ void World::ComputeStaticAnalysis()
         const int rowIndex = y * width;
         for ( int x = 0; x < width; ++x ) {
             const int index = rowIndex + x;
-            const Maps::Tiles & tile = vec_tiles[index];
+            const Maps::Tile & tile = vec_tiles[index];
             MapRegionNode & node = data[ConvertExtendedIndex( index, extendedWidth )];
 
             node.index = index;
             node.passable = tile.GetPassable();
             node.isWater = tile.isWater();
 
-            const MP2::MapObjectType objectType = tile.GetObject();
+            const MP2::MapObjectType objectType = tile.getMainObjectType();
             node.mapObject = MP2::isInGameActionObject( objectType, node.isWater ) ? objectType : 0;
             if ( node.passable != 0 ) {
                 node.type = REGION_NODE_OPEN;
